import urllib

import pytest
import requests
from pytestshellutils.utils import ports
from werkzeug.wrappers import Response  # pylint: disable=3rd-party-module-not-gated

import salt.utils.http as http
from tests.support.mock import MagicMock, patch


def test_requests_session_verify_ssl_false(ssl_webserver, integration_files_dir):
    """
    test salt.utils.http.session when using verify_ssl
    """
    for verify in [True, False, None]:
        kwargs = {"verify_ssl": verify}
        if verify is None:
            kwargs.pop("verify_ssl")

        if verify is True or verify is None:
            with pytest.raises(requests.exceptions.SSLError) as excinfo:
                session = http.session(**kwargs)
                ret = session.get(ssl_webserver.url("this.txt"))
        else:
            session = http.session(**kwargs)
            ret = session.get(ssl_webserver.url("this.txt"))
            assert ret.status_code == 200


def test_session_ca_bundle_verify_false():
    """
    test salt.utils.http.session when using
    both ca_bunlde and verify_ssl false
    """
    ret = http.session(ca_bundle="/tmp/test_bundle", verify_ssl=False)
    assert ret is False


def test_session_headers():
    """
    test salt.utils.http.session when setting
    headers
    """
    ret = http.session(headers={"Content-Type": "application/json"})
    assert ret.headers["Content-Type"] == "application/json"


def test_session_ca_bundle():
    """
    test salt.utils.https.session when setting ca_bundle
    """
    fpath = "/tmp/test_bundle"
    patch_os = patch("os.path.exists", MagicMock(return_value=True))
    with patch_os:
        ret = http.session(ca_bundle=fpath)
    assert ret.verify == fpath


def test_sanitize_url_hide_fields_none():
    """
    Tests sanitizing a url when the hide_fields kwarg is None.
    """
    mock_url = "https://api.testing.com/?&foo=bar&test=testing"
    ret = http.sanitize_url(mock_url, hide_fields=None)
    assert ret == mock_url


def test_sanitize_url_no_elements():
    """
    Tests sanitizing a url when no elements should be sanitized.
    """
    mock_url = "https://api.testing.com/?&foo=bar&test=testing"
    ret = http.sanitize_url(mock_url, [""])
    assert ret == mock_url


def test_sanitize_url_single_element():
    """
    Tests sanitizing a url with only a single element to be sanitized.
    """
    mock_url = (
        "https://api.testing.com/?&keep_it_secret=abcdefghijklmn"
        "&api_action=module.function"
    )
    mock_ret = (
        "https://api.testing.com/?&keep_it_secret=XXXXXXXXXX&"
        "api_action=module.function"
    )
    ret = http.sanitize_url(mock_url, ["keep_it_secret"])
    assert ret == mock_ret


def test_sanitize_url_multiple_elements():
    """
    Tests sanitizing a url with multiple elements to be sanitized.
    """
    mock_url = (
        "https://api.testing.com/?rootPass=badpassword%21"
        "&skipChecks=True&api_key=abcdefghijklmn"
        "&NodeID=12345&api_action=module.function"
    )
    mock_ret = (
        "https://api.testing.com/?rootPass=XXXXXXXXXX"
        "&skipChecks=True&api_key=XXXXXXXXXX"
        "&NodeID=12345&api_action=module.function"
    )
    ret = http.sanitize_url(mock_url, ["api_key", "rootPass"])
    assert ret == mock_ret


# _sanitize_components tests


def test_sanitize_components_no_elements():
    """
    Tests when zero elements need to be sanitized.
    """
    mock_component_list = ["foo=bar", "bar=baz", "hello=world"]
    mock_ret = "foo=bar&bar=baz&hello=world&"
    ret = http._sanitize_url_components(mock_component_list, "api_key")
    assert ret == mock_ret


def test_sanitize_components_one_element():
    """
    Tests a single component to be sanitized.
    """
    mock_component_list = ["foo=bar", "api_key=abcdefghijklmnop"]
    mock_ret = "foo=bar&api_key=XXXXXXXXXX&"
    ret = http._sanitize_url_components(mock_component_list, "api_key")
    assert ret == mock_ret


def test_sanitize_components_multiple_elements():
    """
    Tests two componenets to be sanitized.
    """
    mock_component_list = ["foo=bar", "foo=baz", "api_key=testing"]
    mock_ret = "foo=XXXXXXXXXX&foo=XXXXXXXXXX&api_key=testing&"
    ret = http._sanitize_url_components(mock_component_list, "foo")
    assert ret == mock_ret


@pytest.mark.slow_test
def test_query_null_response():
    """
    This tests that we get a null response when raise_error=False and the
    host/port cannot be reached.
    """
    host = "127.0.0.1"

    port = ports.get_unused_localhost_port()

    url = f"http://{host}:{port}/"
    result = http.query(url, raise_error=False)
    assert result == {"body": None}, result


def test_query_error_handling():
    ret = http.query("http://127.0.0.1:0")
    assert isinstance(ret, dict)
    assert isinstance(ret.get("error", None), str)
    ret = http.query("http://myfoobardomainthatnotexist")
    assert isinstance(ret, dict)
    assert isinstance(ret.get("error", None), str)


def test_parse_cookie_header():
    header = "; ".join(
        [
            "foo=bar",
            "expires=Mon, 03-Aug-20 14:26:27 GMT",
            "path=/",
            "domain=.mydomain.tld",
            "HttpOnly",
            "SameSite=Lax",
            "Secure",
        ]
    )
    ret = http.parse_cookie_header(header)
    cookie = ret.pop(0)
    assert cookie.name == "foo", cookie.name
    assert cookie.value == "bar", cookie.value
    assert cookie.expires == 1596464787, cookie.expires
    assert cookie.path == "/", cookie.path
    assert cookie.domain == ".mydomain.tld", cookie.domain
    assert cookie.secure
    # Only one cookie should have been returned, if anything is left in the
    # parse_cookie_header return then something went wrong.
    assert not ret


@pytest.mark.requires_network
def test_requests_multipart_formdata_post(httpserver):
    """
    Test handling of a multipart/form-data POST using the requests backend
    """
    match_this = (
        "{0}\r\nContent-Disposition: form-data;"
        ' name="fieldname_here"\r\n\r\nmydatahere\r\n{0}--\r\n'
    )

    def mirror_post_handler(request):
        return Response(request.data)

    httpserver.expect_request(
        "/multipart_form_data",
    ).respond_with_handler(mirror_post_handler)
    url = httpserver.url_for("/multipart_form_data")

    ret = http.query(
        url,
        method="POST",
        data="mydatahere",
        formdata=True,
        formdata_fieldname="fieldname_here",
        backend="requests",
    )
    body = ret.get("body", "")
    boundary = body[: body.find("\r")]
    assert body == match_this.format(boundary)


def test_query_proxy(httpserver):
    """
    Test http.query with tornado and with proxy opts set
    and then test with no_proxy set to ensure we dont
    run into issue #55192 again.
    """
    data = "mydatahere"
    opts = {
        "proxy_host": "127.0.0.1",
        "proxy_port": 88,
        "proxy_username": "salt_test",
        "proxy_password": "super_secret",
    }

    with patch("requests.Session") as mock_session:
        mock_session.return_value = MagicMock()
        ret = http.query(
            "https://fake_url",
            method="POST",
            data=data,
            backend="tornado",
            opts=opts,
        )

        assert mock_session.return_value.proxies == {
            "http": "http://salt_test:super_secret@127.0.0.1:88",
            "https": "http://salt_test:super_secret@127.0.0.1:88",
        }

    opts["no_proxy"] = [httpserver.host]

    httpserver.expect_request(
        "/no_proxy_test",
    ).respond_with_data(data)
    url = httpserver.url_for("/no_proxy_test")

    with patch("requests.Session") as mock_session:
        mock_session.return_value = MagicMock()
        ret = http.query(
            url,
            method="POST",
            data=data,
            backend="tornado",
            opts=opts,
        )
        assert not isinstance(mock_session.return_value.proxies, dict)

    ret = http.query(url, method="POST", data=data, backend="tornado", opts=opts)
    body = ret.get("body", "")
    assert body == data


@pytest.mark.parametrize("backend", ["requests", "tornado", "urllib2"])
def test_backends_decode_body_false(httpserver, backend):
    """
    test all backends when using
    decode_body=False that it returns
    bytes and does not try to decode
    """
    url = "/test-bytes"
    data = b"test-bytes"
    httpserver.expect_request(
        url,
    ).respond_with_data(data, content_type="application/octet-stream")
    ret = http.query(
        httpserver.url_for(url),
        backend=backend,
        decode_body=False,
    )
    body = ret.get("body", "")
    assert isinstance(body, bytes)


@pytest.mark.parametrize("backend", ["requests", "tornado", "urllib2"])
def test_backends_decode_body_true(httpserver, backend):
    """
    test all backends when using decode_body=True that it returns string and decodes it.
    """
    url = "/test-decoded-bytes"
    data = b"test-decoded-bytes"
    httpserver.expect_request(
        url,
    ).respond_with_data(data, content_type="application/octet-stream")
    ret = http.query(
        httpserver.url_for(url),
        backend=backend,
    )
    body = ret.get("body", "")
    assert isinstance(body, str)


def test_requests_post_content_type(httpserver):
    url = httpserver.url_for("/post-content-type")
    data = urllib.parse.urlencode({"payload": "test"})
    opts = {
        "proxy_host": "127.0.0.1",
        "proxy_port": 88,
    }
    with patch("requests.Session") as mock_session:
        sess = MagicMock()
        sess.headers = {}
        mock_session.return_value = sess
        ret = http.query(
            url,
            method="POST",
            data=data,
            backend="tornado",
            opts=opts,
        )
        assert "Content-Type" in sess.headers
        assert sess.headers["Content-Type"] == "application/x-www-form-urlencoded"
        assert "Content-Length" in sess.headers
        assert sess.headers["Content-Length"] == "12"



"""
Test utility methods that the idem module and state share
"""

from contextlib import contextmanager

import pytest

import salt.utils.idem as idem
import salt.utils.path
from tests.support.case import TestCase

HAS_IDEM = not salt.utils.path.which("idem")


@pytest.mark.skipif(not idem.HAS_POP[0], reason=str(idem.HAS_POP[1]))
@contextmanager
class TestIdem(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.hub = idem.hub()

    def test_loop(self):
        assert hasattr(self.hub.pop, "Loop")

    def test_subs(self):
        for sub in ("acct", "config", "idem", "exec", "states"):
            with self.subTest(sub=sub):
                assert hasattr(self.hub, sub)

    @pytest.mark.skipif(not HAS_IDEM, reason="idem is not installed")
    def test_idem_ex(self):
        assert hasattr(self.hub.idem, "ex")

    @pytest.mark.skipif(not HAS_IDEM, reason="idem is not installed")
    def test_idem_state_apply(self):
        assert hasattr(self.hub.idem.state, "apply")

    @pytest.mark.skipif(not HAS_IDEM, reason="idem is not installed")
    def test_idem_exec(self):
        # self.hub.exec.test.ping() causes a pylint error because of "exec" in the namespace
        assert getattr(self.hub, "exec").test.ping()

    @pytest.mark.skipif(not HAS_IDEM, reason="idem is not installed")
    def test_idem_state(self):
        ret = self.hub.states.test.succeed_without_changes({}, "test_state")
        assert ret["result"] is True

    def test_config(self):
        assert self.hub.OPT.acct
        assert self.hub.OPT.idem



import logging
import time

import pytest

pytestmark = [
    pytest.mark.skip_unless_on_linux(reason="Only supported on Linux family"),
]

log = logging.getLogger(__name__)


def test_salt_systemd_disabled_preservation(
    call_cli, install_salt_systemd, salt_systemd_setup
):
    """
    Test upgrade of Salt packages preserve disabled state of systemd
    """
    if not install_salt_systemd.upgrade:
        pytest.skip("Not testing an upgrade, do not run")

    # ensure known state, disabled
    test_list = ["salt-api", "salt-minion", "salt-master"]
    for test_item in test_list:
        test_cmd = f"systemctl disable {test_item}"
        ret = call_cli.run("--local", "cmd.run", test_cmd)
        assert ret.returncode == 0

    # Upgrade Salt (inc. minion, master, etc.) from previous version and test
    # pylint: disable=pointless-statement
    install_salt_systemd.install(upgrade=True)
    time.sleep(60)  # give it some time

    # test for disabled systemd state
    test_list = ["salt-api", "salt-minion", "salt-master"]
    for test_item in test_list:
        test_cmd = f"systemctl show -p UnitFileState {test_item}"
        ret = call_cli.run("--local", "cmd.run", test_cmd)
        test_enabled = ret.stdout.strip().split("=")[1].split('"')[0].strip()
        assert ret.returncode == 0
        assert test_enabled == "disabled"


def test_salt_systemd_enabled_preservation(
    call_cli, install_salt_systemd, salt_systemd_setup
):
    """
    Test upgrade of Salt packages preserve enabled state of systemd
    """
    if not install_salt_systemd.upgrade:
        pytest.skip("Not testing an upgrade, do not run")
    install_salt_systemd.no_uninstall = False

    # Upgrade Salt (inc. minion, master, etc.) from previous version and test
    # pylint: disable=pointless-statement
    install_salt_systemd.install(upgrade=True)
    time.sleep(10)  # give it some time

    # test for enabled systemd state
    test_list = ["salt-api", "salt-minion", "salt-master"]
    for test_item in test_list:
        test_cmd = f"systemctl show -p UnitFileState {test_item}"
        ret = call_cli.run("--local", "cmd.run", test_cmd)
        test_enabled = ret.stdout.strip().split("=")[1].split('"')[0].strip()
        assert ret.returncode == 0
        assert test_enabled == "enabled"


def test_salt_systemd_masked_preservation(
    call_cli, install_salt_systemd, salt_systemd_setup, salt_systemd_mask_services
):
    """
    Test upgrade of Salt packages preserves masked state of systemd services
    """
    if not install_salt_systemd.upgrade:
        pytest.skip("Not testing an upgrade, do not run")
    install_salt_systemd.no_uninstall = False

    # Upgrade Salt (inc. minion, master, etc.) from previous version and test
    # pylint: disable=pointless-statement
    install_salt_systemd.install(upgrade=True)
    time.sleep(60)  # give it some time

    # test for masked systemd state
    test_list = ["salt-api", "salt-minion", "salt-master"]
    for test_item in test_list:
        test_cmd = f"systemctl show -p UnitFileState {test_item}"
        ret = call_cli.run("--local", "cmd.run", test_cmd)
        test_masked = ret.stdout.strip().split("=")[1].split('"')[0].strip()
        assert ret.returncode == 0
        assert test_masked == "masked"



import pytest

from tests.conftest import FIPS_TESTRUN


def test_grains_overwrite(salt_cli, salt_master, salt_minion):
    assert not salt_minion.config.get("grains_deep_merge", False)
    # Force a grains sync
    salt_cli.run("saltutil.sync_grains", minion_tgt=salt_minion.id)

    # XXX: This should no longer be neede because of using salt_cli.run.
    # To avoid a race condition on Windows, we need to make sure the
    # `test_custom_grain2.py` file is present in the _grains directory
    # before trying to get the grains. This test may execute before the
    # minion has finished syncing down the files it needs.
    # module = os.path.join(
    #    salt_minion.config["cachedir"],
    #    "files",
    #    "base",
    #    "_grains",
    #    "custom_grain2.py",
    # )
    # assert os.path.exists(module)

    # Check that custom grains are overwritten
    ret = salt_cli.run("grains.items", minion_tgt=salt_minion.id)
    assert ret.data["a_custom"] == {"k2": "v2"}


def test_grains_merge(salt_cli, salt_master):
    minion = salt_master.salt_minion_daemon(
        "test_grains_merge",
        overrides={
            "grains_deep_merge": True,
            # Grains in the minon config won't get merged.
            # "grains": {"a_custom": {"k1": "v1"}},
            "fips_mode": FIPS_TESTRUN,
            "encryption_algorithm": "OAEP-SHA224" if FIPS_TESTRUN else "OAEP-SHA1",
            "signing_algorithm": "PKCS1v15-SHA224" if FIPS_TESTRUN else "PKCS1v15-SHA1",
        },
    )
    minion.after_terminate(
        pytest.helpers.remove_stale_minion_key, salt_master, minion.id
    )
    content = """
    def grain():
        return {"a_custom": {"k1": "v1"}}
    """
    with salt_master.state_tree.base.temp_file("_grains/tempcustom.py", content):
        with minion.started():
            salt_cli.run("saltutil.sync_grains", minion_tgt=minion.id)
            ret = salt_cli.run("grains.item", "a_custom", minion_tgt=minion.id)
            assert ret.data["a_custom"] == {"k1": "v1", "k2": "v2"}



import pathlib

import pytest

import salt.modules.win_file as win_file
import salt.modules.win_lgpo_reg as lgpo_reg
import salt.utils.files
import salt.utils.win_dacl
import salt.utils.win_lgpo_reg
import salt.utils.win_reg
from salt.exceptions import SaltInvocationError

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
    pytest.mark.destructive_test,
]


@pytest.fixture
def configure_loader_modules():
    return {
        win_file: {
            "__utils__": {
                "dacl.set_perms": salt.utils.win_dacl.set_perms,
                "dacl.set_permissions": salt.utils.win_dacl.set_permissions,
            },
        },
    }


@pytest.fixture
def empty_reg_pol_mach():
    class_info = salt.utils.win_lgpo_reg.CLASS_INFO
    reg_pol_file = pathlib.Path(class_info["Machine"]["policy_path"])
    if not reg_pol_file.parent.exists():
        reg_pol_file.parent.mkdir(parents=True)
    with salt.utils.files.fopen(str(reg_pol_file), "wb") as f:
        f.write(salt.utils.win_lgpo_reg.REG_POL_HEADER.encode("utf-16-le"))
    salt.utils.win_reg.delete_key_recursive(hive="HKLM", key="SOFTWARE\\MyKey1")
    salt.utils.win_reg.delete_key_recursive(hive="HKLM", key="SOFTWARE\\MyKey2")
    yield
    salt.utils.win_reg.delete_key_recursive(hive="HKLM", key="SOFTWARE\\MyKey1")
    salt.utils.win_reg.delete_key_recursive(hive="HKLM", key="SOFTWARE\\MyKey2")
    with salt.utils.files.fopen(str(reg_pol_file), "wb") as f:
        f.write(salt.utils.win_lgpo_reg.REG_POL_HEADER.encode("utf-16-le"))


@pytest.fixture
def empty_reg_pol_user():
    class_info = salt.utils.win_lgpo_reg.CLASS_INFO
    reg_pol_file = pathlib.Path(class_info["User"]["policy_path"])
    if not reg_pol_file.parent.exists():
        reg_pol_file.parent.mkdir(parents=True)
    with salt.utils.files.fopen(str(reg_pol_file), "wb") as f:
        f.write(salt.utils.win_lgpo_reg.REG_POL_HEADER.encode("utf-16-le"))
    salt.utils.win_reg.delete_key_recursive(hive="HKCU", key="SOFTWARE\\MyKey1")
    salt.utils.win_reg.delete_key_recursive(hive="HKCU", key="SOFTWARE\\MyKey2")
    yield
    salt.utils.win_reg.delete_key_recursive(hive="HKCU", key="SOFTWARE\\MyKey1")
    salt.utils.win_reg.delete_key_recursive(hive="HKCU", key="SOFTWARE\\MyKey2")
    with salt.utils.files.fopen(str(reg_pol_file), "wb") as f:
        f.write(salt.utils.win_lgpo_reg.REG_POL_HEADER.encode("utf-16-le"))


@pytest.fixture
def reg_pol_mach():
    data_to_write = {
        "SOFTWARE\\MyKey1": {
            "MyValue1": {
                "data": "squidward",
                "type": "REG_SZ",
            },
            "**del.MyValue2": {
                "data": " ",
                "type": "REG_SZ",
            },
        },
        "SOFTWARE\\MyKey2": {
            "MyValue3": {
                "data": ["spongebob", "squarepants"],
                "type": "REG_MULTI_SZ",
            },
        },
    }
    lgpo_reg.write_reg_pol(data_to_write)
    salt.utils.win_reg.set_value(
        hive="HKLM",
        key="SOFTWARE\\MyKey1",
        vname="MyValue1",
        vdata="squidward",
        vtype="REG_SZ",
    )
    salt.utils.win_reg.set_value(
        hive="HKLM",
        key="SOFTWARE\\MyKey2",
        vname="MyValue3",
        vdata=["spongebob", "squarepants"],
        vtype="REG_MULTI_SZ",
    )
    yield
    salt.utils.win_reg.delete_key_recursive(hive="HKLM", key="SOFTWARE\\MyKey1")
    salt.utils.win_reg.delete_key_recursive(hive="HKLM", key="SOFTWARE\\MyKey2")
    class_info = salt.utils.win_lgpo_reg.CLASS_INFO
    reg_pol_file = class_info["Machine"]["policy_path"]
    with salt.utils.files.fopen(reg_pol_file, "wb") as f:
        f.write(salt.utils.win_lgpo_reg.REG_POL_HEADER.encode("utf-16-le"))


@pytest.fixture
def reg_pol_user():
    data_to_write = {
        "SOFTWARE\\MyKey1": {
            "MyValue1": {
                "data": "squidward",
                "type": "REG_SZ",
            },
            "**del.MyValue2": {
                "data": " ",
                "type": "REG_SZ",
            },
        },
        "SOFTWARE\\MyKey2": {
            "MyValue3": {
                "data": ["spongebob", "squarepants"],
                "type": "REG_MULTI_SZ",
            },
        },
    }
    lgpo_reg.write_reg_pol(data_to_write, policy_class="User")
    salt.utils.win_reg.set_value(
        hive="HKCU",
        key="SOFTWARE\\MyKey1",
        vname="MyValue1",
        vdata="squidward",
        vtype="REG_SZ",
    )
    assert salt.utils.win_reg.value_exists(
        hive="HKCU", key="SOFTWARE\\MyKey1", vname="MyValue1"
    )
    salt.utils.win_reg.set_value(
        hive="HKCU",
        key="SOFTWARE\\MyKey2",
        vname="MyValue3",
        vdata=["spongebob", "squarepants"],
        vtype="REG_MULTI_SZ",
    )
    assert salt.utils.win_reg.value_exists(
        hive="HKCU", key="SOFTWARE\\MyKey2", vname="MyValue3"
    )
    yield
    salt.utils.win_reg.delete_key_recursive(hive="HKCU", key="SOFTWARE\\MyKey1")
    salt.utils.win_reg.delete_key_recursive(hive="HKCU", key="SOFTWARE\\MyKey2")
    assert not salt.utils.win_reg.key_exists(hive="HKCU", key="SOFTWARE\\MyKey1")
    assert not salt.utils.win_reg.key_exists(hive="HKCU", key="SOFTWARE\\MyKey2")
    class_info = salt.utils.win_lgpo_reg.CLASS_INFO
    reg_pol_file = class_info["User"]["policy_path"]
    with salt.utils.files.fopen(reg_pol_file, "wb") as f:
        f.write(salt.utils.win_lgpo_reg.REG_POL_HEADER.encode("utf-16-le"))


def test_invalid_policy_class_delete_value():
    pytest.raises(
        SaltInvocationError,
        lgpo_reg.delete_value,
        key="",
        v_name="",
        policy_class="Invalid",
    )


def test_invalid_policy_class_disable_value():
    pytest.raises(
        SaltInvocationError,
        lgpo_reg.disable_value,
        key="",
        v_name="",
        policy_class="Invalid",
    )


def test_invalid_policy_class_get_key():
    pytest.raises(SaltInvocationError, lgpo_reg.get_key, key="", policy_class="Invalid")


def test_invalid_policy_class_get_value():
    pytest.raises(
        SaltInvocationError,
        lgpo_reg.get_value,
        key="",
        v_name="",
        policy_class="Invalid",
    )


def test_invalid_policy_class_read_reg_pol():
    pytest.raises(SaltInvocationError, lgpo_reg.read_reg_pol, policy_class="Invalid")


def test_invalid_policy_class_set_value():
    pytest.raises(
        SaltInvocationError,
        lgpo_reg.set_value,
        key="",
        v_name="",
        v_data="",
        policy_class="Invalid",
    )


def test_invalid_policy_class_write_reg_pol():
    pytest.raises(
        SaltInvocationError, lgpo_reg.write_reg_pol, data={}, policy_class="Invalid"
    )


def test_set_value_invalid_reg_type():
    pytest.raises(
        SaltInvocationError,
        lgpo_reg.set_value,
        key="",
        v_name="",
        v_data="",
        v_type="REG_INVALID",
    )


def test_set_value_invalid_reg_sz():
    pytest.raises(
        SaltInvocationError,
        lgpo_reg.set_value,
        key="",
        v_name="",
        v_data=[],
        v_type="REG_SZ",
    )


def test_set_value_invalid_reg_multi_sz():
    pytest.raises(
        SaltInvocationError,
        lgpo_reg.set_value,
        key="",
        v_name="",
        v_data=1,
        v_type="REG_MULTI_SZ",
    )


def test_set_value_invalid_reg_dword():
    pytest.raises(
        SaltInvocationError, lgpo_reg.set_value, key="", v_name="", v_data="string"
    )


def test_mach_read_reg_pol(empty_reg_pol_mach):
    expected = {}
    result = lgpo_reg.read_reg_pol()
    assert result == expected


def test_mach_write_reg_pol(empty_reg_pol_mach):
    data_to_write = {
        r"SOFTWARE\MyKey": {
            "MyValue": {
                "data": "string",
                "type": "REG_SZ",
            },
        },
    }
    lgpo_reg.write_reg_pol(data_to_write)
    result = lgpo_reg.read_reg_pol()
    assert result == data_to_write


@pytest.mark.parametrize(
    "name,expected",
    [
        ("MyValue", {}),
        ("MyValue1", {"data": "squidward", "type": "REG_SZ"}),
        ("MyValue2", {"data": "**del.MyValue2", "type": "REG_SZ"}),
    ],
)
def test_mach_get_value(reg_pol_mach, name, expected):
    result = lgpo_reg.get_value(key="SOFTWARE\\MyKey1", v_name=name)
    assert result == expected


def test_mach_get_key(reg_pol_mach):
    expected = {
        "MyValue3": {
            "data": ["spongebob", "squarepants"],
            "type": "REG_MULTI_SZ",
        },
    }
    result = lgpo_reg.get_key(key="SOFTWARE\\MyKey2")
    assert result == expected


def test_mach_set_value(empty_reg_pol_mach):
    key = "SOFTWARE\\MyKey"
    v_name = "MyValue"
    # Test command return
    result = lgpo_reg.set_value(key=key, v_name=v_name, v_data="1")
    assert result is True
    # Test value actually set in Registry.pol
    expected = {"data": 1, "type": "REG_DWORD"}
    result = lgpo_reg.get_value(key=key, v_name=v_name)
    assert result == expected
    # Test that the registry value has been set
    expected = {
        "hive": "HKLM",
        "key": key,
        "vname": v_name,
        "vdata": 1,
        "vtype": "REG_DWORD",
        "success": True,
    }
    result = salt.utils.win_reg.read_value(hive="HKLM", key=key, vname=v_name)
    assert result == expected


def test_mach_set_value_existing_change(reg_pol_mach):
    expected = {"data": 1, "type": "REG_DWORD"}
    key = "SOFTWARE\\MyKey"
    v_name = "MyValue1"
    lgpo_reg.set_value(key=key, v_name=v_name, v_data="1")
    result = lgpo_reg.get_value(key=key, v_name=v_name)
    assert result == expected
    expected = {
        "hive": "HKLM",
        "key": key,
        "vname": v_name,
        "vdata": 1,
        "vtype": "REG_DWORD",
        "success": True,
    }
    result = salt.utils.win_reg.read_value(hive="HKLM", key=key, vname=v_name)
    assert result == expected


def test_mach_set_value_existing_no_change(reg_pol_mach):
    expected = {"data": "squidward", "type": "REG_SZ"}
    key = "SOFTWARE\\MyKey"
    v_name = "MyValue1"
    lgpo_reg.set_value(key=key, v_name=v_name, v_data="squidward", v_type="REG_SZ")
    result = lgpo_reg.get_value(key=key, v_name=v_name)
    assert result == expected


def test_mach_disable_value(reg_pol_mach):
    key = "SOFTWARE\\MyKey1"
    # Test that the command completed successfully
    result = lgpo_reg.disable_value(key=key, v_name="MyValue1")
    assert result is True
    # Test that the value was actually set in Registry.pol
    expected = {
        "**del.MyValue1": {"data": " ", "type": "REG_SZ"},
        "**del.MyValue2": {"data": " ", "type": "REG_SZ"},
    }
    result = lgpo_reg.get_key(key=key)
    assert result == expected
    # Test that the registry value has been removed
    result = salt.utils.win_reg.value_exists(hive="HKLM", key=key, vname="MyValue1")
    assert result is False


def test_mach_disable_value_no_change(reg_pol_mach):
    expected = {
        "MyValue1": {"data": "squidward", "type": "REG_SZ"},
        "**del.MyValue2": {"data": " ", "type": "REG_SZ"},
    }
    key = "SOFTWARE\\MyKey1"
    lgpo_reg.disable_value(key=key, v_name="MyValue2")
    result = lgpo_reg.get_key(key=key)
    assert result == expected


def test_mach_delete_value_existing(reg_pol_mach):
    key = "SOFTWARE\\MyKey1"
    # Test that the command completes successfully
    result = lgpo_reg.delete_value(key=key, v_name="MyValue1")
    assert result is True
    # Test that the value is actually removed from Registry.pol
    expected = {
        "**del.MyValue2": {
            "data": " ",
            "type": "REG_SZ",
        },
    }
    result = lgpo_reg.get_key(key=key)
    assert result == expected
    # Test that the registry entry has been removed
    result = salt.utils.win_reg.value_exists(hive="HKLM", key=key, vname="MyValue2")
    assert result is False


def test_mach_delete_value_no_change(empty_reg_pol_mach):
    expected = {}
    key = "SOFTWARE\\MyKey1"
    lgpo_reg.delete_value(key=key, v_name="MyValue2")
    result = lgpo_reg.get_key(key=key)
    assert result == expected


def test_user_read_reg_pol(empty_reg_pol_user):
    expected = {}
    result = lgpo_reg.read_reg_pol(policy_class="User")
    assert result == expected


def test_user_write_reg_pol(empty_reg_pol_user):
    data_to_write = {
        r"SOFTWARE\MyKey": {
            "MyValue": {
                "data": "string",
                "type": "REG_SZ",
            },
        },
    }
    lgpo_reg.write_reg_pol(data_to_write, policy_class="User")
    result = lgpo_reg.read_reg_pol(policy_class="User")
    assert result == data_to_write


@pytest.mark.parametrize(
    "name,expected",
    [
        ("MyValue", {}),
        ("MyValue1", {"data": "squidward", "type": "REG_SZ"}),
        ("MyValue2", {"data": "**del.MyValue2", "type": "REG_SZ"}),
    ],
)
def test_user_get_value(reg_pol_user, name, expected):
    result = lgpo_reg.get_value(
        key="SOFTWARE\\MyKey1",
        v_name=name,
        policy_class="User",
    )
    assert result == expected


def test_user_get_key(reg_pol_user):
    expected = {
        "MyValue3": {
            "data": ["spongebob", "squarepants"],
            "type": "REG_MULTI_SZ",
        },
    }
    result = lgpo_reg.get_key(key="SOFTWARE\\MyKey2", policy_class="User")
    assert result == expected


def test_user_set_value(empty_reg_pol_user):
    key = "SOFTWARE\\MyKey"
    v_name = "MyValue"
    # Test command return
    result = lgpo_reg.set_value(
        key=key,
        v_name=v_name,
        v_data="1",
        policy_class="User",
    )
    assert result is True
    # Test value actually set in Registry.pol
    expected = {"data": 1, "type": "REG_DWORD"}
    result = lgpo_reg.get_value(key=key, v_name=v_name, policy_class="User")
    assert result == expected
    # Test that the registry value has not been set
    result = salt.utils.win_reg.value_exists(hive="HKCU", key=key, vname=v_name)
    assert result is False


def test_user_set_value_existing_change(reg_pol_user):
    expected = {"data": 1, "type": "REG_DWORD"}
    key = "SOFTWARE\\MyKey"
    v_name = "MyValue1"
    lgpo_reg.set_value(key=key, v_name=v_name, v_data="1", policy_class="User")
    result = lgpo_reg.get_value(key=key, v_name=v_name, policy_class="User")
    assert result == expected
    # Test that the registry value has not been set
    result = salt.utils.win_reg.value_exists(hive="HKCU", key=key, vname=v_name)
    assert result is False


def test_user_set_value_existing_no_change(reg_pol_user):
    expected = {"data": "squidward", "type": "REG_SZ"}
    key = "SOFTWARE\\MyKey"
    v_name = "MyValue1"
    lgpo_reg.set_value(
        key=key,
        v_name=v_name,
        v_data="squidward",
        v_type="REG_SZ",
        policy_class="User",
    )
    result = lgpo_reg.get_value(key=key, v_name=v_name, policy_class="User")
    assert result == expected


def test_user_disable_value(reg_pol_user):
    key = "SOFTWARE\\MyKey1"
    # Test that the command completed successfully
    result = lgpo_reg.disable_value(key=key, v_name="MyValue1", policy_class="User")
    assert result is True
    # Test that the value was actually set in Registry.pol
    expected = {
        "**del.MyValue1": {"data": " ", "type": "REG_SZ"},
        "**del.MyValue2": {"data": " ", "type": "REG_SZ"},
    }
    result = lgpo_reg.get_key(key=key, policy_class="User")
    assert result == expected
    # Test that the registry value has not been removed
    result = salt.utils.win_reg.value_exists(hive="HKCU", key=key, vname="MyValue1")
    assert result is True


def test_user_disable_value_no_change(reg_pol_user):
    expected = {
        "MyValue1": {"data": "squidward", "type": "REG_SZ"},
        "**del.MyValue2": {"data": " ", "type": "REG_SZ"},
    }
    key = "SOFTWARE\\MyKey1"
    lgpo_reg.disable_value(key=key, v_name="MyValue2", policy_class="User")
    result = lgpo_reg.get_key(key=key, policy_class="User")
    assert result == expected


def test_user_delete_value_existing(reg_pol_user):
    key = "SOFTWARE\\MyKey1"
    # Test that the command completes successfully
    result = lgpo_reg.delete_value(key=key, v_name="MyValue1", policy_class="User")
    assert result is True
    # Test that the value is actually removed from Registry.pol
    expected = {
        "**del.MyValue2": {
            "data": " ",
            "type": "REG_SZ",
        },
    }
    result = lgpo_reg.get_key(key=key, policy_class="User")
    assert result == expected
    # Test that the registry entry has not been removed
    result = salt.utils.win_reg.value_exists(hive="HKCU", key=key, vname="MyValue1")
    assert result is True


def test_user_delete_value_no_change(empty_reg_pol_user):
    expected = {}
    key = "SOFTWARE\\MyKey1"
    lgpo_reg.delete_value(key=key, v_name="MyValue2", policy_class="User")
    result = lgpo_reg.get_key(key=key, policy_class="User")
    assert result == expected



"""
Configuration of the GNOME desktop
========================================

Control the GNOME settings

.. code-block:: yaml

    localdesktop_wm_prefs:
        gnomedesktop.wm_preferences:
            - user: username
            - audible_bell: false
            - action_double_click_titlebar: 'toggle-maximize'
            - visual_bell: true
            - num_workspaces: 6
    localdesktop_lockdown:
        gnomedesktop.desktop_lockdown:
            - user: username
            - disable_user_switching: true
    localdesktop_interface:
        gnomedesktop.desktop_interface:
            - user: username
            - clock_show_date: true
            - clock_format: 12h
"""

import logging
import re

log = logging.getLogger(__name__)


def _check_current_value(gnome_kwargs, value):
    """
    Check the current value with the passed value
    """
    current_value = __salt__["gnome.get"](**gnome_kwargs)
    return str(current_value) == str(value)


def _do(name, gnome_kwargs, preferences):
    """
    worker function for the others to use
    this handles all the gsetting magic
    """
    ret = {"name": name, "result": True, "changes": {}, "comment": ""}

    messages = []

    for pref in preferences:
        key = pref
        value = preferences[pref]

        if isinstance(value, bool):
            ftype = "boolean"

            # need to convert boolean values to strings and make lowercase to
            # pass to gsettings
            value = str(value).lower()

        elif isinstance(value, int):
            ftype = "int"
        elif isinstance(value, str):
            ftype = "string"
        else:
            ftype = "string"

        gnome_kwargs.update({"key": key, "value": value})

        if _check_current_value(gnome_kwargs, value):
            messages.append(f"{key} is already set to {value}")
        else:
            result = __salt__["gnome.set"](**gnome_kwargs)
            if result["retcode"] == 0:
                messages.append(f"Setting {key} to {value}")
                ret["changes"][key] = f"{key}:{value}"
                ret["result"] = True
            else:
                messages.append(result["stdout"])
                ret["result"] = False

        ret["comment"] = ", ".join(messages)

    return ret


def wm_preferences(
    name,
    user=None,
    action_double_click_titlebar=None,
    action_middle_click_titlebar=None,
    action_right_click_titlebar=None,
    application_based=None,
    audible_bell=None,
    auto_raise=None,
    auto_raise_delay=None,
    button_layout=None,
    disable_workarounds=None,
    focus_mode=None,
    focus_new_windows=None,
    mouse_button_modifier=None,
    num_workspaces=None,
    raise_on_click=None,
    resize_with_right_button=None,
    theme=None,
    titlebar_font=None,
    titlebar_uses_system_font=None,
    visual_bell=None,
    visual_bell_type=None,
    workspace_names=None,
    **kwargs,
):
    """
    wm_preferences: sets values in the org.gnome.desktop.wm.preferences schema
    """
    gnome_kwargs = {"user": user, "schema": "org.gnome.desktop.wm.preferences"}

    preferences = [
        "action_double_click_titlebar",
        "action_middle_click_titlebar",
        "action_right_click_titlebar",
        "application_based",
        "audible_bell",
        "auto_raise",
        "auto_raise_delay",
        "button_layout",
        "disable_workarounds",
        "focus_mode",
        "focus_new_windows",
        "mouse_button_modifier",
        "num_workspaces",
        "raise_on_click",
        "resize_with_right_button",
        "theme",
        "titlebar_font",
        "titlebar_uses_system_font",
        "visual_bell",
        "visual_bell_type",
        "workspace_names",
    ]

    preferences_hash = {}
    for pref in preferences:
        if pref in locals() and locals()[pref] is not None:
            key = re.sub("_", "-", pref)
            preferences_hash[key] = locals()[pref]

    return _do(name, gnome_kwargs, preferences_hash)


def desktop_lockdown(
    name,
    user=None,
    disable_application_handlers=None,
    disable_command_line=None,
    disable_lock_screen=None,
    disable_log_out=None,
    disable_print_setup=None,
    disable_printing=None,
    disable_save_to_disk=None,
    disable_user_switching=None,
    user_administration_disabled=None,
    **kwargs,
):
    """
    desktop_lockdown: sets values in the org.gnome.desktop.lockdown schema
    """
    gnome_kwargs = {"user": user, "schema": "org.gnome.desktop.lockdown"}

    preferences = [
        "disable_application_handlers",
        "disable_command_line",
        "disable_lock_screen",
        "disable_log_out",
        "disable_print_setup",
        "disable_printing",
        "disable_save_to_disk",
        "disable_user_switching",
        "user_administration_disabled",
    ]

    preferences_hash = {}
    for pref in preferences:
        if pref in locals() and locals()[pref] is not None:
            key = re.sub("_", "-", pref)
            preferences_hash[key] = locals()[pref]

    return _do(name, gnome_kwargs, preferences_hash)


def desktop_interface(
    name,
    user=None,
    automatic_mnemonics=None,
    buttons_have_icons=None,
    can_change_accels=None,
    clock_format=None,
    clock_show_date=None,
    clock_show_seconds=None,
    cursor_blink=None,
    cursor_blink_time=None,
    cursor_blink_timeout=None,
    cursor_size=None,
    cursor_theme=None,
    document_font_name=None,
    enable_animations=None,
    font_name=None,
    gtk_color_palette=None,
    gtk_color_scheme=None,
    gtk_im_module=None,
    gtk_im_preedit_style=None,
    gtk_im_status_style=None,
    gtk_key_theme=None,
    gtk_theme=None,
    gtk_timeout_initial=None,
    gtk_timeout_repeat=None,
    icon_theme=None,
    menubar_accel=None,
    menubar_detachable=None,
    menus_have_icons=None,
    menus_have_tearoff=None,
    monospace_font_name=None,
    show_input_method_menu=None,
    show_unicode_menu=None,
    text_scaling_factor=None,
    toolbar_detachable=None,
    toolbar_icons_size=None,
    toolbar_style=None,
    toolkit_accessibility=None,
    **kwargs,
):
    """
    desktop_interface: sets values in the org.gnome.desktop.interface schema
    """
    gnome_kwargs = {"user": user, "schema": "org.gnome.desktop.interface"}

    preferences = [
        "automatic_mnemonics",
        "buttons_have_icons",
        "can_change_accels",
        "clock_format",
        "clock_show_date",
        "clock_show_seconds",
        "cursor_blink",
        "cursor_blink_time",
        "cursor_blink_timeout",
        "cursor_size",
        "cursor_theme",
        "document_font_name",
        "enable_animations",
        "font_name",
        "gtk_color_palette",
        "gtk_color_scheme",
        "gtk_im_module",
        "gtk_im_preedit_style",
        "gtk_im_status_style",
        "gtk_key_theme",
        "gtk_theme",
        "gtk_timeout_initial",
        "gtk_timeout_repeat",
        "icon_theme",
        "menubar_accel",
        "menubar_detachable",
        "menus_have_icons",
        "menus_have_tearoff",
        "monospace_font_name",
        "show_input_method_menu",
        "show_unicode_menu",
        "text_scaling_factor",
        "toolbar_detachable",
        "toolbar_icons_size",
        "toolbar_style",
        "toolkit_accessibility",
    ]

    preferences_hash = {}
    for pref in preferences:
        if pref in locals() and locals()[pref] is not None:
            key = re.sub("_", "-", pref)
            preferences_hash[key] = locals()[pref]

    return _do(name, gnome_kwargs, preferences_hash)



"""
Pull sdb values from a YAML file

:maintainer:    SaltStack
:maturity:      New
:platform:      all

.. versionadded:: 2017.7.0

Configuration:

.. code-block:: yaml

    my-yaml-file:
      driver: yaml
      files:
        - /path/to/foo.yaml
        - /path/to/bar.yaml

The files are merged together and the result is searched using the same
mechanism Salt uses for searching Grains and Pillar data structures.

Optional configuration:

.. code-block:: yaml

    my-yaml-file:
      driver: yaml
      files:
        - /path/to/foo.yaml
        - /path/to/bar.yaml
      merge:
        strategy: smart
        merge_list: false
      gpg: true

.. versionadded:: 2018.3.0

Setting the ``gpg`` option to ``true`` (default is ``false``) will decrypt
embedded GPG-encrypted data using the :py:mod:`GPG renderer
<salt.renderers.gpg>`.
"""

import logging

import salt.exceptions
import salt.loader
import salt.renderers.gpg
import salt.utils.data
import salt.utils.dictupdate
import salt.utils.files

log = logging.getLogger(__name__)

__func_alias__ = {"set_": "set"}


def __virtual__():
    return True


def set_(*args, **kwargs):  # pylint: disable=W0613
    """
    Setting a value is not supported; edit the YAML files directly
    """
    raise salt.exceptions.NotImplemented()


def get(key, profile=None):  # pylint: disable=W0613
    """
    Get a value from the dictionary
    """
    data = _get_values(profile)

    # Decrypt SDB data if specified in the profile
    if profile and profile.get("gpg", False):
        return salt.utils.data.traverse_dict_and_list(_decrypt(data), key, None)

    return salt.utils.data.traverse_dict_and_list(data, key, None)


def _get_values(profile=None):
    """
    Retrieve all the referenced files, deserialize, then merge them together
    """
    profile = profile or {}
    serializers = salt.loader.serializers(__opts__)

    ret = {}
    for fname in profile.get("files", []):
        try:
            with salt.utils.files.flopen(fname) as yamlfile:
                contents = serializers.yaml.deserialize(yamlfile)
                ret = salt.utils.dictupdate.merge(
                    ret, contents, **profile.get("merge", {})
                )
        except OSError:
            log.error("File '%s' not found ", fname)
        except TypeError as exc:
            log.error("Error deserializing sdb file '%s': %s", fname, exc)
    return ret


def _decrypt(data):
    """
    Pass the dictionary through the GPG renderer to decrypt encrypted values.
    """
    return salt.loader.render(__opts__, __salt__)["gpg"](data)



"""
Vault Pillar Module

:maintainer:    SaltStack
:maturity:      New
:platform:      all

.. versionadded:: 2016.11.0

This module allows pillar data to be stored in Hashicorp Vault.

Base configuration instructions are documented in the :ref:`execution module docs <vault-setup>`.
Below are noted extra configuration required for the pillar module, but the base
configuration must also be completed.

After the base Vault configuration is created, add the configuration below to
the ext_pillar section in the Salt master configuration.

.. code-block:: yaml

    ext_pillar:
      - vault: path=secret/salt

Each key needs to have all the key-value pairs with the names you
require. Avoid naming every key 'password' as you they will collide:

If you want to nest results under a nesting_key name use the following format:

.. code-block:: yaml

    ext_pillar:
      - vault:
          conf: path=secret/salt
          nesting_key: vault_key_name

.. code-block:: bash

    $ vault write secret/salt auth=my_password master=127.0.0.1

The above will result in two pillars being available, ``auth`` and ``master``.

You can then use normal pillar requests to get each key pair directly from
pillar root. Example:

.. code-block:: bash

    $ salt-ssh '*' pillar.get auth

Multiple Vault sources may also be used:

.. code-block:: yaml

    ext_pillar:
      - vault: path=secret/salt
      - vault: path=secret/root
      - vault: path=secret/minions/{minion}/pass
      - vault: path=secret/roles/{pillar[roles]}/pass

You can also use nesting here as well.  Identical nesting keys will get merged.

.. code-block:: yaml

    ext_pillar:
      - vault:
           conf: path=secret/salt
           nesting_key: keyname1
      - vault:
           conf: path=secret/root
           nesting_key: keyname1
      - vault:
           conf: path=secret/minions/{minion}/pass
           nesting_key: keyname2

The difference between the return with and without the nesting key is shown below.
This example takes the key value pairs returned from vault as follows:

.. code-block:: text

    path=secret/salt

    Key             Value
    ---             -----
    salt-passwd     badpasswd1

    path=secret/root

    Key             Value
    ---             -----
    root-passwd     rootbadpasswd1

    path=secret/minions/{minion}/pass

    Key             Value
    ---             -----
    minion-passwd   minionbadpasswd1


.. code-block:: yaml

    #Nesting Key not defined

    local:
        ----------
        salt-passwd:
            badpasswd1
        root-passwd:
            rootbadpasswd1
        minion-passwd:
            minionbadpasswd1

    #Nesting Key defined

    local:
        ----------
        keyname1:
            ----------
                salt-passwd:
                    badpasswd1
                root-passwd:
                    rootbadpasswd1
        keyname2:
            ----------
                minion-passwd:
                    minionbadpasswd1

.. versionadded:: 3006.0

    Pillar values from previously rendered pillars can be used to template
    vault ext_pillar paths.

Using pillar values to template vault pillar paths requires them to be defined
before the vault ext_pillar is called. Especially consider the significancy
of :conf_master:`ext_pillar_first <ext_pillar_first>` master config setting.

If a pillar pattern matches multiple paths, the results are merged according to
the master configuration values :conf_master:`pillar_source_merging_strategy <pillar_source_merging_strategy>`
and :conf_master:`pillar_merge_lists <pillar_merge_lists>` by default.

If the optional nesting_key was defined, the merged result will be nested below.
There is currently no way to nest multiple results under different keys.

You can override the merging behavior per defined ext_pillar:

.. code-block:: yaml

    ext_pillar:
      - vault:
           conf: path=secret/roles/{pillar[roles]}
           merge_strategy: smart
           merge_lists: false
"""

import logging

from requests.exceptions import HTTPError

import salt.utils.dictupdate

log = logging.getLogger(__name__)


def __virtual__():
    """
    This module has no external dependencies
    """
    return True


def ext_pillar(
    minion_id,  # pylint: disable=W0613
    pillar,  # pylint: disable=W0613
    conf,
    nesting_key=None,
    merge_strategy=None,
    merge_lists=None,
    extra_minion_data=None,
):
    """
    Get pillar data from Vault for the configuration ``conf``.
    """
    extra_minion_data = extra_minion_data or {}
    if extra_minion_data.get("_vault_runner_is_compiling_pillar_templates"):
        # Disable vault ext_pillar while compiling pillar for vault policy templates
        return {}

    comps = conf.split()

    paths = [comp for comp in comps if comp.startswith("path=")]
    if not paths:
        log.error('"%s" is not a valid Vault ext_pillar config', conf)
        return {}

    merge_strategy = merge_strategy or __opts__.get(
        "pillar_source_merging_strategy", "smart"
    )
    merge_lists = merge_lists or __opts__.get("pillar_merge_lists", False)
    vault_pillar = {}

    path_pattern = paths[0].replace("path=", "")
    for path in _get_paths(path_pattern, minion_id, pillar):
        try:
            version2 = __utils__["vault.is_v2"](path)
            if version2["v2"]:
                path = version2["data"]

            url = f"v1/{path}"
            response = __utils__["vault.make_request"]("GET", url)
            response.raise_for_status()
            vault_pillar_single = response.json().get("data", {})

            if vault_pillar_single and version2["v2"]:
                vault_pillar_single = vault_pillar_single["data"]

            vault_pillar = salt.utils.dictupdate.merge(
                vault_pillar,
                vault_pillar_single,
                strategy=merge_strategy,
                merge_lists=merge_lists,
            )
        except HTTPError:
            log.info("Vault secret not found for: %s", path)

    if nesting_key:
        vault_pillar = {nesting_key: vault_pillar}
    return vault_pillar


def _get_paths(path_pattern, minion_id, pillar):
    """
    Get the paths that should be merged into the pillar dict
    """
    mappings = {"minion": minion_id, "pillar": pillar}

    paths = []
    try:
        for expanded_pattern in __utils__["vault.expand_pattern_lists"](
            path_pattern, **mappings
        ):
            paths.append(expanded_pattern.format(**mappings))
    except KeyError:
        log.warning("Could not resolve pillar path pattern %s", path_pattern)

    log.debug("%s vault pillar paths: %s", minion_id, paths)
    return paths



import pytest
from pytestskipmarkers.utils import platform


@pytest.mark.skip_unless_on_linux(reason="Linux test only")
def test_services(install_salt, salt_call_cli):
    """
    Check if Services are enabled/disabled
    """
    services_disabled = []
    services_enabled = []
    if install_salt.distro_id in ("ubuntu", "debian"):
        services_enabled = ["salt-master", "salt-minion", "salt-syndic", "salt-api"]
    elif install_salt.distro_id in (
        "almalinux",
        "rocky",
        "centos",
        "redhat",
        "amzn",
        "fedora",
    ):
        services_disabled = ["salt-master", "salt-minion", "salt-syndic", "salt-api"]
    elif install_salt.distro_id == "photon":
        services_enabled = ["salt-master", "salt-minion", "salt-syndic", "salt-api"]
    elif platform.is_darwin():
        services_enabled = ["salt-minion"]
        services_disabled = []
    else:
        pytest.fail(f"Don't know how to handle os_family={install_salt.distro_id}")

    for service in services_enabled:
        test_cmd = f"systemctl show -p UnitFileState {service}"
        ret = salt_call_cli.run("--local", "cmd.run", test_cmd)
        test_enabled = ret.stdout.strip().split("=")[1].split('"')[0].strip()
        assert ret.returncode == 0
        assert test_enabled == "enabled"

    for service in services_disabled:
        test_cmd = f"systemctl show -p UnitFileState {service}"
        ret = salt_call_cli.run("--local", "cmd.run", test_cmd)
        test_enabled = ret.stdout.strip().split("=")[1].split('"')[0].strip()
        assert ret.returncode == 0
        assert test_enabled == "disabled"



"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

from collections import OrderedDict

import pytest

import salt.states.test as test
from salt.exceptions import SaltInvocationError
from tests.support.mock import MagicMock, patch


@pytest.fixture
def configure_loader_modules():
    return {test: {"__low__": {"__reqs__": {"watch": ""}}}}


def test_succeed_without_changes():
    """
    Test to returns successful.
    """
    ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
    with patch.dict(test.__opts__, {"test": False}):
        ret.update({"comment": "Success!"})
        assert test.succeed_without_changes("salt") == ret

    with patch.dict(test.__opts__, {"test": False}):
        ret.update({"comment": "A success comment!"})
        assert test.succeed_without_changes("salt", comment="A success comment!") == ret


def test_fail_without_changes():
    """
    Test to returns failure.
    """
    ret = {"name": "salt", "changes": {}, "result": False, "comment": ""}
    with patch.dict(test.__opts__, {"test": False}):
        ret.update({"comment": "Failure!"})
        assert test.fail_without_changes("salt") == ret

    with patch.dict(test.__opts__, {"test": False}):
        ret.update({"comment": "A failure comment!"})
        assert test.fail_without_changes("salt", comment="A failure comment!") == ret

    with patch.dict(test.__opts__, {"test": True}):
        ret.update({"comment": "If we weren't testing, this would be a failure!"})
        assert test.fail_without_changes("salt") == ret


def test_succeed_with_changes():
    """
    Test to returns successful and changes is not empty
    """
    ret = {"name": "salt", "changes": {}, "result": False, "comment": ""}
    with patch.dict(test.__opts__, {"test": False}):
        ret.update(
            {
                "changes": {
                    "testing": {
                        "new": "Something pretended to change",
                        "old": "Unchanged",
                    }
                },
                "comment": "Success!",
                "result": True,
            }
        )
        assert test.succeed_with_changes("salt") == ret

    with patch.dict(test.__opts__, {"test": False}):
        ret.update(
            {
                "changes": {
                    "testing": {
                        "new": "Something pretended to change",
                        "old": "Unchanged",
                    }
                },
                "comment": "A success comment!",
                "result": True,
            }
        )
        assert test.succeed_with_changes("salt", comment="A success comment!") == ret

    with patch.dict(test.__opts__, {"test": True}):
        ret.update(
            {
                "changes": {
                    "testing": {
                        "new": "Something pretended to change",
                        "old": "Unchanged",
                    }
                },
                "comment": (
                    "If we weren't testing, this would be successful with changes"
                ),
                "result": None,
            }
        )
        assert test.succeed_with_changes("salt", comment="A success comment!") == ret


def test_fail_with_changes():
    """
    Test to returns failure and changes is not empty.
    """
    ret = {"name": "salt", "changes": {}, "result": False, "comment": ""}
    with patch.dict(test.__opts__, {"test": False}):
        ret.update(
            {
                "changes": {
                    "testing": {
                        "new": "Something pretended to change",
                        "old": "Unchanged",
                    }
                },
                "comment": "Failure!",
                "result": False,
            }
        )
        assert test.fail_with_changes("salt") == ret

    with patch.dict(test.__opts__, {"test": False}):
        ret.update(
            {
                "changes": {
                    "testing": {
                        "new": "Something pretended to change",
                        "old": "Unchanged",
                    }
                },
                "comment": "A failure comment!",
                "result": False,
            }
        )
        assert test.fail_with_changes("salt", comment="A failure comment!") == ret

    with patch.dict(test.__opts__, {"test": True}):
        ret.update(
            {
                "changes": {
                    "testing": {
                        "new": "Something pretended to change",
                        "old": "Unchanged",
                    }
                },
                "comment": "If we weren't testing, this would be failed with changes",
                "result": None,
            }
        )
        assert test.fail_with_changes("salt", comment="A failure comment!") == ret


def test_configurable_test_state():
    """
    Test test.configurable_test_state with and without comment
    """
    mock_name = "cheese_shop"
    mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
    mock_changes = {
        "testing": {"old": "Unchanged", "new": "Something pretended to change"}
    }

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": True,
            "comment": "",
        }
        ret = test.configurable_test_state(mock_name)
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": True,
            "comment": mock_comment,
        }
        ret = test.configurable_test_state(mock_name, comment=mock_comment)
        assert ret == mock_ret


def test_configurable_test_state_changes():
    """
    Test test.configurable_test_state with permutations of changes and with
    comment
    """
    mock_name = "cheese_shop"
    mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
    mock_changes = {
        "testing": {"old": "Unchanged", "new": "Something pretended to change"}
    }

    with patch.dict(test.__opts__, {"test": False}):
        ret = test.configurable_test_state(
            mock_name, changes="Random", comment=mock_comment
        )
        assert ret["name"] == mock_name
        assert ret["changes"] in [mock_changes, {}]
        assert ret["result"] is True
        assert ret["comment"] == mock_comment

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": True,
            "comment": mock_comment,
        }
        ret = test.configurable_test_state(
            mock_name, changes=True, comment=mock_comment
        )
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": {},
            "result": True,
            "comment": mock_comment,
        }
        ret = test.configurable_test_state(
            mock_name, changes=False, comment=mock_comment
        )
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": False}):
        pytest.raises(
            SaltInvocationError,
            test.configurable_test_state,
            mock_name,
            changes="Cheese",
        )


def test_configurable_test_state_result():
    """
    Test test.configurable_test_state with permutations of result and with
    comment
    """
    mock_name = "cheese_shop"
    mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
    mock_changes = {
        "testing": {"old": "Unchanged", "new": "Something pretended to change"}
    }

    with patch.dict(test.__opts__, {"test": False}):
        ret = test.configurable_test_state(
            mock_name, result="Random", comment=mock_comment
        )
        assert ret["name"] == mock_name
        assert ret["changes"] == mock_changes
        assert ret["result"] in [True, False]
        assert ret["comment"] == mock_comment

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": True,
            "comment": mock_comment,
        }
        ret = test.configurable_test_state(mock_name, result=True, comment=mock_comment)
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": False,
            "comment": mock_comment,
        }
        ret = test.configurable_test_state(
            mock_name, result=False, comment=mock_comment
        )
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": False}):
        pytest.raises(
            SaltInvocationError,
            test.configurable_test_state,
            mock_name,
            result="Cheese",
        )


def test_configurable_test_state_warnings():
    """
    Test test.configurable_test_state with and without warnings
    """
    # Configure mock parameters
    mock_name = "cheese_shop"
    mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
    mock_warning = "Today the van broke down."
    mock_warning_list = [mock_warning, "Oooooooooohhh........!"]
    mock_changes = {
        "testing": {"old": "Unchanged", "new": "Something pretended to change"}
    }

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": True,
            "comment": "",
        }
        ret = test.configurable_test_state(mock_name)
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": True,
            "comment": "",
            "warnings": mock_warning_list,
        }
        ret = test.configurable_test_state(mock_name, warnings=mock_warning_list)
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": False}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": True,
            "comment": "",
            "warnings": ["Today the van broke down."],
        }
        ret = test.configurable_test_state(mock_name, warnings=mock_warning)

        assert ret == mock_ret


def test_configurable_test_state_test():
    """
    Test test.configurable_test_state with test=True with and without
    comment
    """
    mock_name = "cheese_shop"
    mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
    mock_changes = {
        "testing": {"old": "Unchanged", "new": "Something pretended to change"}
    }

    with patch.dict(test.__opts__, {"test": True}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": None,
            "comment": "This is a test",
        }
        ret = test.configurable_test_state(mock_name)
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": True}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": None,
            "comment": mock_comment,
        }
        ret = test.configurable_test_state(mock_name, comment=mock_comment)
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": True}):
        mock_ret = {
            "name": mock_name,
            "changes": mock_changes,
            "result": None,
            "comment": mock_comment,
        }
        ret = test.configurable_test_state(
            mock_name, changes=True, comment=mock_comment
        )
        assert ret == mock_ret

    with patch.dict(test.__opts__, {"test": True}):
        mock_ret = {
            "name": mock_name,
            "changes": {},
            "result": True,
            "comment": mock_comment,
        }
        ret = test.configurable_test_state(
            mock_name, changes=False, comment=mock_comment
        )
        assert ret == mock_ret


def test_mod_watch():
    """
    Test to call this function via a watch statement
    """
    ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
    ret.update(
        {
            "changes": {"Requisites with changes": []},
            "comment": "Watch statement fired.",
        }
    )
    assert test.mod_watch("salt") == ret


def test_check_pillar_present():
    """
    Test to ensure the check_pillar function works properly with the 'present'
    keyword in the absence of a 'type' keyword.
    """
    ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
    pillar_return = "I am a pillar."
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert test.check_pillar("salt", present="my_pillar") == ret


def test_check_pillar_string():
    """
    Test to ensure the check_pillar function works properly with the 'key_type'
    checks, using the string key_type.
    """
    ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
    pillar_return = "I am a pillar."
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert test.check_pillar("salt", string="my_pillar") == ret
    pillar_return = "I am a pillar."
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert test.check_pillar("salt", string="my_pillar") == ret
    pillar_return = {"this": "dictionary"}
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert not test.check_pillar("salt", string="my_pillar")["result"]
    pillar_return = ["I am a pillar."]
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert not test.check_pillar("salt", string="my_pillar")["result"]
    # With a boolean
    pillar_return = True
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert not test.check_pillar("salt", string="my_pillar")["result"]
    pillar_return = 1
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert not test.check_pillar("salt", string="my_pillar")["result"]


def test_check_pillar_dictionary():
    """
    Test to ensure the check_pillar function works properly with the 'key_type'
    checks, using the dictionary key_type.
    """
    ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
    pillar_return = {"this": "dictionary"}
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert test.check_pillar("salt", dictionary="my_pillar") == ret
    pillar_return = OrderedDict({"this": "dictionary"})
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert test.check_pillar("salt", dictionary="my_pillar") == ret
    pillar_return = "I am a pillar."
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert not test.check_pillar("salt", dictionary="my_pillar")["result"]
    pillar_return = ["I am a pillar."]
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert not test.check_pillar("salt", dictionary="my_pillar")["result"]
    pillar_return = True
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert not test.check_pillar("salt", dictionary="my_pillar")["result"]
    pillar_return = 1
    pillar_mock = MagicMock(return_value=pillar_return)
    with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
        assert not test.check_pillar("salt", dictionary="my_pillar")["result"]



import pytest

import salt.config
import salt.loader


@pytest.fixture
def matchers(minion_opts):
    return salt.loader.matchers(minion_opts)


def test_sanity(matchers):
    match = matchers["confirm_top.confirm_top"]
    assert match("*", []) is True



import pytest

from tests.support.case import ModuleCase


class SysctlModuleTest(ModuleCase):
    def setUp(self):
        super().setUp()
        ret = self.run_function("cmd.has_exec", ["sysctl"])
        if not ret:
            self.skipTest("sysctl not found")

    def test_show(self):
        ret = self.run_function("sysctl.show")
        self.assertIsInstance(ret, dict, "sysctl.show return wrong type")
        self.assertGreater(len(ret), 10, "sysctl.show return few data")

    @pytest.mark.skip_unless_on_linux
    def test_show_linux(self):
        ret = self.run_function("sysctl.show")
        self.assertIn("kernel.ostype", ret, "kernel.ostype absent")

    @pytest.mark.skip_unless_on_freebsd
    def test_show_freebsd(self):
        ret = self.run_function("sysctl.show")
        self.assertIn("vm.vmtotal", ret, "Multiline variable absent")
        self.assertGreater(
            len(ret.get("vm.vmtotal").splitlines()),
            1,
            "Multiline value was parsed wrong",
        )

    @pytest.mark.skip_unless_on_openbsd
    def test_show_openbsd(self):
        ret = self.run_function("sysctl.show")
        self.assertIn("kern.ostype", ret, "kern.ostype absent")
        self.assertEqual(ret.get("kern.ostype"), "OpenBSD", "Incorrect kern.ostype")

    @pytest.mark.skip_unless_on_darwin
    def test_show_darwin(self):
        ret = self.run_function("sysctl.show")
        self.assertIn("kern.ostype", ret, "kern.ostype absent")
        self.assertEqual(ret.get("kern.ostype"), "Darwin", "Incorrect kern.ostype")



import os
import shutil

import pytest

try:
    import pythoncom
    from win32com.shell import shell

    HAS_WIN32 = True
except ImportError:
    HAS_WIN32 = False

pytestmark = [
    pytest.mark.windows_whitelisted,
    pytest.mark.skip_unless_on_windows,
    pytest.mark.skipif(not HAS_WIN32, reason="Requires Win32 libraries"),
]


@pytest.fixture(scope="module")
def shortcut(states):
    return states.shortcut


@pytest.fixture(scope="module")
def shortcut_mod(modules):
    return modules.shortcut


@pytest.fixture(scope="function")
def tmp_dir(tmp_path_factory):
    test_dir = tmp_path_factory.mktemp("test_dir")
    yield test_dir
    if test_dir.exists():
        shutil.rmtree(str(test_dir))


@pytest.fixture(scope="function")
def tmp_shortcut(tmp_path_factory):
    tmp_dir = tmp_path_factory.mktemp("test_dir")
    tmp_shortcut = tmp_dir / "test.lnk"
    # https://docs.microsoft.com/en-us/windows/win32/api/shobjidl_core/nn-shobjidl_core-ishelllinkw
    # http://timgolden.me.uk/python/win32_how_do_i/create-a-shortcut.html
    short_cut = pythoncom.CoCreateInstance(
        shell.CLSID_ShellLink,
        None,
        pythoncom.CLSCTX_INPROC_SERVER,
        shell.IID_IShellLink,
    )
    program = r"C:\Windows\notepad.exe"
    short_cut.SetArguments("existing arguments")
    short_cut.SetDescription("existing description")
    short_cut.SetIconLocation(program, 0)
    short_cut.SetHotkey(1601)
    short_cut.SetPath(program)
    short_cut.SetShowCmd(1)
    short_cut.SetWorkingDirectory(os.path.dirname(program))

    persist_file = short_cut.QueryInterface(pythoncom.IID_IPersistFile)
    persist_file.Save(str(tmp_shortcut), 0)

    yield tmp_shortcut

    if tmp_dir.exists():
        shutil.rmtree(str(tmp_dir))


def test_present(shortcut, shortcut_mod, tmp_dir):
    file_shortcut = tmp_dir / "test.lnk"
    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="present arguments",
        description="present description",
        hot_key="Ctrl+Shift+D",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Maximized",
        working_dir=r"C:\Windows",
    )
    assert ret.name == str(file_shortcut)
    assert ret.changes == {}
    assert "Shortcut created" in ret.comment
    assert ret.result is True

    expected = {
        "arguments": "present arguments",
        "description": "present description",
        "hot_key": "Ctrl+Shift+D",
        "icon_index": 0,
        "icon_location": r"C:\Windows\notepad.exe",
        "path": str(file_shortcut),
        "target": r"C:\Windows\notepad.exe",
        "window_style": "Maximized",
        "working_dir": r"C:\Windows",
    }
    results = shortcut_mod.get(path=str(file_shortcut))
    assert results == expected


def test_present_existing_same(shortcut, tmp_shortcut):
    file_shortcut = str(tmp_shortcut)

    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="existing arguments",
        description="existing description",
        hot_key="Alt+Ctrl+A",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Normal",
        working_dir=r"C:\Windows",
    )

    assert ret.name == str(file_shortcut)
    assert ret.changes == {}
    assert "Shortcut already present and configured" in ret.comment
    assert ret.result is True


def test_present_existing(shortcut, tmp_shortcut):
    file_shortcut = str(tmp_shortcut)

    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="present arguments",
        description="present description",
        hot_key="Ctrl+Shift+D",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Maximized",
        working_dir=r"C:\Windows",
    )

    assert ret.name == str(file_shortcut)
    assert ret.changes == {}
    assert "Found existing shortcut" in ret.comment
    assert ret.result is False


def test_present_existing_force(shortcut, shortcut_mod, tmp_shortcut):
    file_shortcut = str(tmp_shortcut)

    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="present arguments",
        description="present description",
        hot_key="Ctrl+Shift+D",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Maximized",
        working_dir=r"C:\Windows",
        force=True,
    )

    changes = {
        "arguments": {"new": "present arguments", "old": "existing arguments"},
        "description": {"new": "present description", "old": "existing description"},
        "hot_key": {"new": "Ctrl+Shift+D", "old": "Alt+Ctrl+A"},
        "window_style": {"new": "Maximized", "old": "Normal"},
    }

    assert ret.name == str(file_shortcut)
    assert ret.changes == changes
    assert "Shortcut modified" in ret.comment
    assert ret.result is True

    expected = {
        "arguments": "present arguments",
        "description": "present description",
        "hot_key": "Ctrl+Shift+D",
        "icon_index": 0,
        "icon_location": r"C:\Windows\notepad.exe",
        "path": str(file_shortcut),
        "target": r"C:\Windows\notepad.exe",
        "window_style": "Maximized",
        "working_dir": r"C:\Windows",
    }
    results = shortcut_mod.get(path=str(file_shortcut))
    assert results == expected


def test_present_existing_backup(shortcut, shortcut_mod, tmp_shortcut):
    file_shortcut = str(tmp_shortcut)

    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="present arguments",
        description="present description",
        hot_key="Ctrl+Shift+D",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Maximized",
        working_dir=r"C:\Windows",
        backup=True,
    )

    changes = {
        "arguments": {"new": "present arguments", "old": "existing arguments"},
        "description": {"new": "present description", "old": "existing description"},
        "hot_key": {"new": "Ctrl+Shift+D", "old": "Alt+Ctrl+A"},
        "window_style": {"new": "Maximized", "old": "Normal"},
    }
    assert ret.name == str(file_shortcut)
    assert ret.changes == changes
    assert "Shortcut modified" in ret.comment
    assert ret.result is True

    expected = {
        "arguments": "present arguments",
        "description": "present description",
        "hot_key": "Ctrl+Shift+D",
        "icon_index": 0,
        "icon_location": r"C:\Windows\notepad.exe",
        "path": str(file_shortcut),
        "target": r"C:\Windows\notepad.exe",
        "window_style": "Maximized",
        "working_dir": r"C:\Windows",
    }
    results = shortcut_mod.get(path=str(file_shortcut))
    assert results == expected


def test_present_existing_subdir(shortcut, tmp_dir):
    file_shortcut = tmp_dir / "subdir" / "test.lnk"
    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="present arguments",
        description="present description",
        hot_key="Ctrl+Shift+D",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Maximized",
        working_dir=r"C:\Windows",
    )
    assert ret.name == str(file_shortcut)
    assert ret.changes == {}
    assert "Failed to create the shortcut" in ret.comment
    assert ret.result is False


def test_present_existing_subdir_make_dirs(shortcut, shortcut_mod, tmp_dir):
    file_shortcut = tmp_dir / "subdir" / "test.lnk"
    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="present arguments",
        description="present description",
        hot_key="Ctrl+Shift+D",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Maximized",
        working_dir=r"C:\Windows",
        make_dirs=True,
    )
    assert ret.name == str(file_shortcut)
    assert ret.changes == {}
    assert "Shortcut created" in ret.comment
    assert ret.result is True

    expected = {
        "arguments": "present arguments",
        "description": "present description",
        "hot_key": "Ctrl+Shift+D",
        "icon_index": 0,
        "icon_location": r"C:\Windows\notepad.exe",
        "path": str(file_shortcut),
        "target": r"C:\Windows\notepad.exe",
        "window_style": "Maximized",
        "working_dir": r"C:\Windows",
    }
    results = shortcut_mod.get(path=str(file_shortcut))
    assert results == expected


def test_present_test_true(shortcut, tmp_dir):
    file_shortcut = tmp_dir / "test.lnk"
    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="present arguments",
        description="present description",
        hot_key="Ctrl+Shift+D",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Maximized",
        working_dir=r"C:\Windows",
        test=True,
    )
    assert ret.name == str(file_shortcut)
    assert ret.changes == {}
    assert "Shortcut will be created" in ret.comment
    assert ret.result is None


def test_present_existing_test_true(shortcut, tmp_shortcut):
    file_shortcut = tmp_shortcut
    ret = shortcut.present(
        name=str(file_shortcut),
        arguments="present arguments",
        description="present description",
        hot_key="Ctrl+Shift+D",
        icon_location=r"C:\Windows\notepad.exe",
        icon_index=0,
        target=r"C:\Windows\notepad.exe",
        window_style="Maximized",
        working_dir=r"C:\Windows",
        test=True,
    )
    changes = {
        "arguments": {"new": "present arguments", "old": "existing arguments"},
        "description": {"new": "present description", "old": "existing description"},
        "hot_key": {"new": "Ctrl+Shift+D", "old": "Alt+Ctrl+A"},
        "window_style": {"new": "Maximized", "old": "Normal"},
    }
    assert ret.name == str(file_shortcut)
    assert ret.changes == changes
    assert "Shortcut will be modified" in ret.comment
    assert ret.result is None



"""
Unit tests for the Vault runner
"""

import logging

import pytest

import salt.runners.vault as vault
from tests.support.mock import MagicMock, Mock, patch

log = logging.getLogger(__name__)


@pytest.fixture
def configure_loader_modules():
    return {vault: {}}


@pytest.fixture
def grains():
    return {
        "id": "test-minion",
        "roles": ["web", "database"],
        "aux": ["foo", "bar"],
        "deep": {"foo": {"bar": {"baz": ["hello", "world"]}}},
        "mixedcase": "UP-low-UP",
    }


@pytest.fixture
def pillar():
    return {
        "role": "test",
    }


@pytest.fixture
def expand_pattern_lists():
    with patch.dict(
        vault.__utils__,
        {
            "vault.expand_pattern_lists": Mock(
                side_effect=lambda x, *args, **kwargs: [x]
            )
        },
    ):
        yield


@pytest.mark.usefixtures("expand_pattern_lists")
def test_get_policies_for_nonexisting_minions():
    minion_id = "salt_master"
    # For non-existing minions, or the master-minion, grains will be None
    cases = {
        "no-tokens-to-replace": ["no-tokens-to-replace"],
        "single-dict:{minion}": [f"single-dict:{minion_id}"],
        "single-grain:{grains[os]}": [],
    }
    with patch(
        "salt.utils.minions.get_minion_data",
        MagicMock(return_value=(None, None, None)),
    ):
        for case, correct_output in cases.items():
            test_config = {"policies": [case]}
            output = vault._get_policies(
                minion_id, test_config
            )  # pylint: disable=protected-access
            diff = set(output).symmetric_difference(set(correct_output))
            if diff:
                log.debug("Test %s failed", case)
                log.debug("Expected:\n\t%s\nGot\n\t%s", output, correct_output)
                log.debug("Difference:\n\t%s", diff)
            assert output == correct_output


@pytest.mark.usefixtures("expand_pattern_lists")
def test_get_policies(grains):
    """
    Ensure _get_policies works as intended.
    The expansion of lists is tested in the vault utility module unit tests.
    """
    cases = {
        "no-tokens-to-replace": ["no-tokens-to-replace"],
        "single-dict:{minion}": ["single-dict:test-minion"],
        "should-not-cause-an-exception,but-result-empty:{foo}": [],
        "Case-Should-Be-Lowered:{grains[mixedcase]}": [
            "case-should-be-lowered:up-low-up"
        ],
    }

    with patch(
        "salt.utils.minions.get_minion_data",
        MagicMock(return_value=(None, grains, None)),
    ):
        for case, correct_output in cases.items():
            test_config = {"policies": [case]}
            output = vault._get_policies(
                "test-minion", test_config
            )  # pylint: disable=protected-access
            diff = set(output).symmetric_difference(set(correct_output))
            if diff:
                log.debug("Test %s failed", case)
                log.debug("Expected:\n\t%s\nGot\n\t%s", output, correct_output)
                log.debug("Difference:\n\t%s", diff)
            assert output == correct_output


@pytest.mark.usefixtures("expand_pattern_lists")
@pytest.mark.parametrize(
    "pattern,count",
    [
        ("salt_minion_{minion}", 0),
        ("salt_grain_{grains[id]}", 0),
        ("unset_{foo}", 0),
        ("salt_pillar_{pillar[role]}", 1),
    ],
)
def test_get_policies_does_not_render_pillar_unnecessarily(
    pattern, count, grains, pillar
):
    """
    The pillar data should only be refreshed in case items are accessed.
    """
    with patch("salt.utils.minions.get_minion_data", autospec=True) as get_minion_data:
        get_minion_data.return_value = (None, grains, None)
        with patch("salt.pillar.get_pillar", autospec=True) as get_pillar:
            get_pillar.return_value.compile_pillar.return_value = pillar
            test_config = {"policies": [pattern]}
            vault._get_policies(
                "test-minion", test_config, refresh_pillar=True
            )  # pylint: disable=protected-access
            assert get_pillar.call_count == count


def test_get_token_create_url():
    """
    Ensure _get_token_create_url parses config correctly
    """
    assert (
        vault._get_token_create_url(  # pylint: disable=protected-access
            {"url": "http://127.0.0.1"}
        )
        == "http://127.0.0.1/v1/auth/token/create"
    )
    assert (
        vault._get_token_create_url(  # pylint: disable=protected-access
            {"url": "https://127.0.0.1/"}
        )
        == "https://127.0.0.1/v1/auth/token/create"
    )
    assert (
        vault._get_token_create_url(  # pylint: disable=protected-access
            {"url": "http://127.0.0.1:8200", "role_name": "therole"}
        )
        == "http://127.0.0.1:8200/v1/auth/token/create/therole"
    )
    assert (
        vault._get_token_create_url(  # pylint: disable=protected-access
            {"url": "https://127.0.0.1/test", "role_name": "therole"}
        )
        == "https://127.0.0.1/test/v1/auth/token/create/therole"
    )



n9k_running_config = """
!Command: show running-config
!Running configuration last done at: Wed Nov 27 18:54:15 2019
!Time: Wed Nov 27 20:21:59 2019

version 9.2(4) Bios:version 07.66
hostname n9k-salt-switch
vdc n9k-salt-switch id 1
  limit-resource vlan minimum 16 maximum 4094
  limit-resource vrf minimum 2 maximum 4096
  limit-resource port-channel minimum 0 maximum 256
  limit-resource u4route-mem minimum 248 maximum 248
  limit-resource u6route-mem minimum 96 maximum 96
  limit-resource m4route-mem minimum 58 maximum 58
  limit-resource m6route-mem minimum 8 maximum 8

feature privilege
feature telnet
feature nxapi
feature bash-shell
cfs eth distribute
feature bgp
feature lldp

username devops password 5 $5$cG3ULyiY$xywJrS7bVTLV2FJy32eA3mVyixfiMqXl5GvnyHMX6Y5  role network-admin
username devops shelltype bash
username devops passphrase  lifetime 99999 warntime 14 gracetime 3
username salt_test password 5 $5$bMOnjQ.w$6ROSiY7Ic7dvfC4jXvQazDrXUH/Cneci371BLC6a.ZD  role network-admin
username salt_test role dev-ops
username salt_test passphrase  lifetime 99999 warntime 14 gracetime 3
username salt_test_2 password 5 $5$jWfBfcJv$LvtViMNcz34fzhD7nfroUa2NPcNbOacLh.WkBXgXocB  role network-operator
username salt_test_2 passphrase  lifetime 99999 warntime 14 gracetime 3
ip domain-lookup
ip name-server 1.1.1.1 2.2.2.2
no system default switchport
system default switchport shutdown
no logging event link-status default
no logging event link-status enable
no logging event trunk-status enable
time-range ans-range
copp profile strict
snmp-server user devops network-admin auth md5 0xbb62c3df89880933539503f7675e8aa8 priv 0xbb62c3df89880933539503f7675e8aa8 localizedkey
snmp-server user salt_test network-admin auth md5 0x3a16a4d0a569be282cc5fbc96138fac8 priv 0x3a16a4d0a569be282cc5fbc96138fac8 localizedkey
rmon event 1 description FATAL(1) owner PMON@FATAL
rmon event 2 description CRITICAL(2) owner PMON@CRITICAL
rmon event 3 description ERROR(3) owner PMON@ERROR
rmon event 4 description WARNING(4) owner PMON@WARNING
rmon event 5 description INFORMATION(5) owner PMON@INFO
no snmp-server enable traps entity entity_mib_change
no snmp-server enable traps entity entity_module_status_change
no snmp-server enable traps entity entity_power_status_change
no snmp-server enable traps entity entity_module_inserted
no snmp-server enable traps entity entity_module_removed
no snmp-server enable traps entity entity_unrecognised_module
no snmp-server enable traps entity entity_fan_status_change
no snmp-server enable traps entity entity_power_out_change
no snmp-server enable traps link linkDown
no snmp-server enable traps link linkUp
no snmp-server enable traps link extended-linkDown
no snmp-server enable traps link extended-linkUp
no snmp-server enable traps link cieLinkDown
no snmp-server enable traps link cieLinkUp
no snmp-server enable traps link delayed-link-state-change
no snmp-server enable traps rf redundancy_framework
no snmp-server enable traps license notify-license-expiry
no snmp-server enable traps license notify-no-license-for-feature
no snmp-server enable traps license notify-licensefile-missing
no snmp-server enable traps license notify-license-expiry-warning
no snmp-server enable traps upgrade UpgradeOpNotifyOnCompletion
no snmp-server enable traps upgrade UpgradeJobStatusNotify
no snmp-server enable traps rmon risingAlarm
no snmp-server enable traps rmon fallingAlarm
no snmp-server enable traps rmon hcRisingAlarm
no snmp-server enable traps rmon hcFallingAlarm
no snmp-server enable traps entity entity_sensor
no snmp-server enable traps entity cefcMIBEnableStatusNotification
no snmp-server enable traps generic coldStart
no snmp-server enable traps generic warmStart
no snmp-server enable traps storm-control cpscEventRev1
no snmp-server enable traps link cErrDisableInterfaceEventRev1
no snmp-server enable traps link cmn-mac-move-notification
ntp server 10.81.254.202 use-vrf management

vlan 1

vrf context management
  ip route 0.0.0.0/0 10.122.197.1
vrf context myvrf
vrf context testing
nxapi http port 80
no nxapi https
nxapi ssl ciphers weak
nxapi ssl protocols TLSv1


interface port-channel10

interface port-channel19

interface Ethernet1/1

interface Ethernet1/2

interface Ethernet1/3

interface Ethernet1/4

interface Ethernet1/5

interface Ethernet1/6

interface Ethernet1/7

interface Ethernet1/8

interface Ethernet1/9

interface Ethernet1/10

interface Ethernet1/11

interface Ethernet1/12

interface Ethernet1/13

interface Ethernet1/14

interface Ethernet1/15

interface Ethernet1/16

interface Ethernet1/17

interface Ethernet1/18

interface Ethernet1/19

interface Ethernet1/20

interface Ethernet1/21

interface Ethernet1/22

interface Ethernet1/23

interface Ethernet1/24

interface Ethernet1/25

interface Ethernet1/26

interface Ethernet1/27

interface Ethernet1/28

interface Ethernet1/29

interface Ethernet1/30

interface Ethernet1/31

interface Ethernet1/32

interface Ethernet1/33

interface Ethernet1/34

interface Ethernet1/35

interface Ethernet1/36

interface Ethernet1/37

interface Ethernet1/38

interface Ethernet1/39

interface Ethernet1/40

interface Ethernet1/41

interface Ethernet1/42

interface Ethernet1/43

interface Ethernet1/44

interface Ethernet1/45

interface Ethernet1/46

interface Ethernet1/47

interface Ethernet1/48

interface Ethernet2/1

interface Ethernet2/2

interface Ethernet2/3

interface Ethernet2/4

interface Ethernet2/5

interface Ethernet2/6

interface Ethernet2/7

interface Ethernet2/8

interface Ethernet2/9

interface Ethernet2/10

interface Ethernet2/11

interface Ethernet2/12

interface mgmt0
  vrf member management
  ip address 192.168.1.5/24
cli alias name wr copy running startup-config
line console
line vty
boot nxos bootflash:/nxos.9.2.4.bin

no logging logfile
no logging monitor
no logging module
no logging console
"""

n9k_show_running_config_list = [
    "\n!Command: show running-config\n!No configuration change since last"
    " restart\n!Time: Tue Dec  3 15:53:49 2019\n\nversion 9.2(4) Bios:version 07.66"
    " \nhostname n9k-device\ninstall feature-set fex\nvdc n9k-device id 1\n  allow"
    " feature-set fex\n  limit-resource vlan minimum 16 maximum 4094\n  limit-resource"
    " vrf minimum 2 maximum 4096\n  limit-resource port-channel minimum 0 maximum 256\n"
    "  limit-resource u4route-mem minimum 248 maximum 248\n  limit-resource u6route-mem"
    " minimum 96 maximum 96\n  limit-resource m4route-mem minimum 58 maximum 58\n "
    " limit-resource m6route-mem minimum 8 maximum 8\nfeature-set fex\n\nfeature"
    " privilege\nfeature tacacs+\ncfs eth distribute\nfeature ngmvpn\nfeature"
    " ospf\nfeature bgp\nfeature pim\nfeature vn-segment-vlan-based\nfeature"
    " lacp\nfeature vpc\nfeature lldp\nfeature bfd\n\nusername admin password 5"
    " $5$mOJsBDrL$lhsuyA8VT/fkWM/5XaZcgHlJIYaaS/J5fh8qdDr6fU3  role"
    " network-admin\nusername devops_user password 5 !  role network-operator\nusername"
    " devops_user passphrase  lifetime 99999 warntime 14 gracetime 3\nusername devops"
    " password 5 $5$.cbtHHuX$IL7tIg97MVeJZcyMhKQ15MsgzVj8w2L1uFufFnaVFv6  role"
    " network-admin\nusername devops passphrase  lifetime 99999 warntime 14 gracetime"
    " 3\nusername salt_test password 5"
    " $5$ALIDDA$epnGyDlACNBu6jqufkSIK26V0zRmO2ANJ6yTXqZwRv8  role"
    " network-operator\nusername salt_test role dev-ops\nusername salt_test role"
    " network-admin\nusername salt_test passphrase  lifetime 99999 warntime 14"
    " gracetime 3\nusername my_test password 5 !  role network-operator\nusername"
    " my_test passphrase  lifetime 99999 warntime 14 gracetime 3\nPassword secure mode"
    " ?? unknown security item\nip domain-lookup\nip name-server 1.1.1.1 2.2.2.2\nno"
    " system default switchport\nsystem default switchport shutdown\nno logging event"
    " link-status default\nno logging event link-status enable\nno logging event"
    " trunk-status enable\ntime-range ans-range\ntime-range my_range\ncopp profile"
    " strict\nsnmp-server user admin network-admin auth md5"
    " 0xbb62c3df89880933539503f7675e8aa8 priv 0xbb62c3df89880933539503f7675e8aa8"
    " localizedkey\nsnmp-server user devops network-admin auth md5"
    " 0xbb62c3df89880933539503f7675e8aa8 priv 0xbb62c3df89880933539503f7675e8aa8"
    " localizedkey\nsnmp-server user devops_user network-operator \nsnmp-server user"
    " my_test network-operator \nsnmp-server user salt_test network-operator auth md5"
    " 0xb3470bc06abb1789337e17e47d048fa2 priv 0xb3470bc06abb1789337e17e47d048fa2"
    " localizedkey\nsnmp-server user salt_test dev-ops\nsnmp-server user salt_test"
    " network-admin\nrmon event 1 description FATAL(1) owner PMON@FATAL\nrmon event 2"
    " description CRITICAL(2) owner PMON@CRITICAL\nrmon event 3 description ERROR(3)"
    " owner PMON@ERROR\nrmon event 4 description WARNING(4) owner PMON@WARNING\nrmon"
    " event 5 description INFORMATION(5) owner PMON@INFO\nntp server 10.81.254.202"
    " use-vrf management\n\nvlan 1\n\nvrf context management\n  ip domain-name cisco\n "
    " ip name-server 192.168.55.247\n  ip route 0.0.0.0/0 192.168.1.84\nvpc domain"
    " 100\n\ninterface port-channel10\n  switchport\n  switchport mode"
    " trunk\n\ninterface port-channel19\n\ninterface port-channel42\n  description"
    " foo\n\ninterface port-channel55\n\ninterface port-channel66\n\ninterface"
    " port-channel100\n  switchport\n  switchport mode trunk\n\ninterface Ethernet1/1\n"
    "  switchport\n  switchport mode trunk\n\ninterface Ethernet1/2\n  switchport\n "
    " switchport mode trunk\n\ninterface Ethernet1/3\n  switchport\n\ninterface"
    " Ethernet1/4\n\ninterface Ethernet1/5\n\ninterface Ethernet1/6\n\ninterface"
    " Ethernet1/7\n\ninterface Ethernet1/8\n\ninterface Ethernet1/9\n\ninterface"
    " Ethernet1/10\n\ninterface Ethernet1/11\n\ninterface Ethernet1/12\n\ninterface"
    " Ethernet1/13\n\ninterface Ethernet1/14\n\ninterface Ethernet1/15\n\ninterface"
    " Ethernet1/16\n\ninterface Ethernet1/17\n\ninterface Ethernet1/18\n\ninterface"
    " Ethernet1/19\n\ninterface Ethernet1/20\n\ninterface Ethernet1/21\n\ninterface"
    " Ethernet1/22\n\ninterface Ethernet1/23\n\ninterface Ethernet1/24\n\ninterface"
    " Ethernet1/25\n\ninterface Ethernet1/26\n\ninterface Ethernet1/27\n\ninterface"
    " Ethernet1/28\n\ninterface Ethernet1/29\n\ninterface Ethernet1/30\n\ninterface"
    " Ethernet1/31\n\ninterface Ethernet1/32\n\ninterface Ethernet1/33\n\ninterface"
    " Ethernet1/34\n\ninterface Ethernet1/35\n\ninterface Ethernet1/36\n\ninterface"
    " Ethernet1/37\n\ninterface Ethernet1/38\n\ninterface Ethernet1/39\n\ninterface"
    " Ethernet1/40\n\ninterface Ethernet1/41\n\ninterface Ethernet1/42\n\ninterface"
    " Ethernet1/43\n\ninterface Ethernet1/44\n\ninterface Ethernet1/45\n\ninterface"
    " Ethernet1/46\n\ninterface Ethernet1/47\n\ninterface Ethernet1/48\n\ninterface"
    " Ethernet2/1\n\ninterface Ethernet2/2\n\ninterface Ethernet2/3\n\ninterface"
    " Ethernet2/4\n\ninterface Ethernet2/5\n\ninterface Ethernet2/6\n\ninterface"
    " Ethernet2/7\n\ninterface Ethernet2/8\n\ninterface Ethernet2/9\n\ninterface"
    " Ethernet2/10\n\ninterface Ethernet2/11\n\ninterface Ethernet2/12\n\ninterface"
    " mgmt0\n  vrf member management\n  ip address 192.168.1.55/24\n\ninterface"
    " loopback0\n  shutdown\n\ninterface loopback1\ncli alias name wr copy running"
    " startup-config\nline console\nline vty\nboot nxos bootflash:/nxos.9.2.4.bin"
    " \n\nno logging logfile\nno logging module\nno logging console\n\n\n"
]

n9k_show_running_inc_username_list = [
    "username salt_test password 5"
    " $5$ALIDDA$epnGyDlACNBu6jqufkSIK26V0zRmO2ANJ6yTXqZwRv8  role network-operator\n"
]



"""
The service module for OpenBSD

.. important::
    If you feel that Salt should be using this module to manage services on a
    minion, and it is using a different module (or gives an error similar to
    *'service.start' is not available*), see :ref:`here
    <module-provider-override>`.
"""

import fnmatch
import logging
import os
import re

import salt.utils.data
import salt.utils.files

log = logging.getLogger(__name__)

# XXX enable/disable support would be nice

# Define the module's virtual name
__virtualname__ = "service"

__func_alias__ = {"reload_": "reload"}


def __virtual__():
    """
    Only work on OpenBSD
    """
    if __grains__["os"] == "OpenBSD" and os.path.exists("/etc/rc.d/rc.subr"):
        krel = list(list(map(int, __grains__["kernelrelease"].split("."))))
        # The -f flag, used to force a script to run even if disabled,
        # was added after the 5.0 release.
        # the rcctl(8) command is the preferred way to manage services.
        if krel[0] > 5 or (krel[0] == 5 and krel[1] > 0):
            if not os.path.exists("/usr/sbin/rcctl"):
                return __virtualname__
    return (
        False,
        "The openbsdservice execution module cannot be loaded: "
        "only available on OpenBSD systems.",
    )


def start(name):
    """
    Start the specified service

    CLI Example:

    .. code-block:: bash

        salt '*' service.start <service name>
    """
    cmd = f"/etc/rc.d/{name} -f start"
    return not __salt__["cmd.retcode"](cmd)


def stop(name):
    """
    Stop the specified service

    CLI Example:

    .. code-block:: bash

        salt '*' service.stop <service name>
    """
    cmd = f"/etc/rc.d/{name} -f stop"
    return not __salt__["cmd.retcode"](cmd)


def restart(name):
    """
    Restart the named service

    CLI Example:

    .. code-block:: bash

        salt '*' service.restart <service name>
    """
    cmd = f"/etc/rc.d/{name} -f restart"
    return not __salt__["cmd.retcode"](cmd)


def status(name, sig=None):
    """
    Return the status for a service.
    If the name contains globbing, a dict mapping service name to True/False
    values is returned.

    .. versionchanged:: 2018.3.0
        The service name can now be a glob (e.g. ``salt*``)

    Args:
        name (str): The name of the service to check
        sig (str): Signature to use to find the service via ps

    Returns:
        bool: True if running, False otherwise
        dict: Maps service name to True if running, False otherwise

    CLI Example:

    .. code-block:: bash

        salt '*' service.status <service name> [service signature]
    """
    if sig:
        return bool(__salt__["status.pid"](sig))

    contains_globbing = bool(re.search(r"\*|\?|\[.+\]", name))
    if contains_globbing:
        services = fnmatch.filter(get_all(), name)
    else:
        services = [name]
    results = {}
    for service in services:
        cmd = f"/etc/rc.d/{service} -f check"
        results[service] = not __salt__["cmd.retcode"](cmd, ignore_retcode=True)
    if contains_globbing:
        return results
    return results[name]


def reload_(name):
    """
    .. versionadded:: 2014.7.0

    Reload the named service

    CLI Example:

    .. code-block:: bash

        salt '*' service.reload <service name>
    """
    cmd = f"/etc/rc.d/{name} -f reload"
    return not __salt__["cmd.retcode"](cmd)


service_flags_regex = re.compile(r"^\s*(\w[\d\w]*)_flags=(?:(NO)|.*)$")
pkg_scripts_regex = re.compile(r"^\s*pkg_scripts=\'(.*)\'$")
start_daemon_call_regex = re.compile(r"(\s*start_daemon(?!\(\)))")
start_daemon_parameter_regex = re.compile(r"(?:\s+(\w[\w\d]*))")


def _get_rc():
    """
    Returns a dict where the key is the daemon's name and
    the value a boolean indicating its status (True: enabled or False: disabled).
    Check the daemons started by the system in /etc/rc and
    configured in /etc/rc.conf and /etc/rc.conf.local.
    Also add to the dict all the localy enabled daemons via $pkg_scripts.
    """
    daemons_flags = {}

    try:
        # now read the system startup script /etc/rc
        # to know what are the system enabled daemons
        with salt.utils.files.fopen("/etc/rc", "r") as handle:
            lines = salt.utils.data.decode(handle.readlines())
    except OSError:
        log.error("Unable to read /etc/rc")
    else:
        for line in lines:
            match = start_daemon_call_regex.match(line)
            if match:
                # the matched line is a call to start_daemon()
                # we remove the function name
                line = line[len(match.group(1)) :]
                # we retrieve each daemon name from the parameters of start_daemon()
                for daemon in start_daemon_parameter_regex.findall(line):
                    # mark it as enabled
                    daemons_flags[daemon] = True

    # this will execute rc.conf and rc.conf.local
    # used in /etc/rc at boot to start the daemons
    variables = __salt__["cmd.run"](
        "(. /etc/rc.conf && set)",
        clean_env=True,
        output_loglevel="quiet",
        python_shell=True,
    ).split("\n")
    for var in variables:
        match = service_flags_regex.match(var)
        if match:
            # the matched var look like daemon_name_flags=, we test its assigned value
            # NO: disabled, everything else: enabled
            # do not create a new key if the service hasn't been found in /etc/rc, see $pkg_scripts
            if match.group(2) == "NO":
                daemons_flags[match.group(1)] = False
        else:
            match = pkg_scripts_regex.match(var)
            if match:
                # the matched var is pkg_scripts
                # we can retrieve the name of each localy enabled daemon that wasn't hand started via /etc/rc
                for daemon in match.group(1).split():
                    # create a new key and mark it as enabled
                    daemons_flags[daemon] = True

    return daemons_flags


def available(name):
    """
    .. versionadded:: 2014.7.0

    Returns ``True`` if the specified service is available, otherwise returns
    ``False``.

    CLI Example:

    .. code-block:: bash

        salt '*' service.available sshd
    """
    path = f"/etc/rc.d/{name}"
    return os.path.isfile(path) and os.access(path, os.X_OK)


def missing(name):
    """
    .. versionadded:: 2014.7.0

    The inverse of service.available.
    Returns ``True`` if the specified service is not available, otherwise returns
    ``False``.

    CLI Example:

    .. code-block:: bash

        salt '*' service.missing sshd
    """
    return not available(name)


def get_all():
    """
    .. versionadded:: 2014.7.0

    Return all available boot services

    CLI Example:

    .. code-block:: bash

        salt '*' service.get_all
    """
    services = []
    if not os.path.isdir("/etc/rc.d"):
        return services
    for service in os.listdir("/etc/rc.d"):
        # this will remove rc.subr and all non executable files
        if available(service):
            services.append(service)
    return sorted(services)


def get_enabled():
    """
    .. versionadded:: 2014.7.0

    Return a list of service that are enabled on boot

    CLI Example:

    .. code-block:: bash

        salt '*' service.get_enabled
    """
    services = []
    for daemon, is_enabled in _get_rc().items():
        if is_enabled:
            services.append(daemon)
    return sorted(set(get_all()) & set(services))


def enabled(name, **kwargs):
    """
    .. versionadded:: 2014.7.0

    Return True if the named service is enabled, false otherwise

    CLI Example:

    .. code-block:: bash

        salt '*' service.enabled <service name>
    """
    return name in get_enabled()


def get_disabled():
    """
    .. versionadded:: 2014.7.0

    Return a set of services that are installed but disabled

    CLI Example:

    .. code-block:: bash

        salt '*' service.get_disabled
    """
    services = []
    for daemon, is_enabled in _get_rc().items():
        if not is_enabled:
            services.append(daemon)
    return sorted(set(get_all()) & set(services))


def disabled(name):
    """
    .. versionadded:: 2014.7.0

    Return True if the named service is disabled, false otherwise

    CLI Example:

    .. code-block:: bash

        salt '*' service.disabled <service name>
    """
    return name in get_disabled()



"""
Send a message to Slack
=======================

This state is useful for sending messages to Slack during state runs.

.. versionadded:: 2015.5.0

.. code-block:: yaml

    slack-message:
      slack.post_message:
        - channel: '#general'
        - from_name: SuperAdmin
        - message: 'This state was executed successfully.'
        - api_key: peWcBiMOS9HrZG15peWcBiMOS9HrZG15

The api key can be specified in the master or minion configuration like below:

.. code-block:: yaml

    slack:
      api_key: peWcBiMOS9HrZG15peWcBiMOS9HrZG15

"""

from salt.exceptions import SaltInvocationError


def __virtual__():
    """
    Only load if the slack module is available in __salt__
    """
    if "slack.post_message" in __salt__:
        return "slack"
    return (False, "slack module could not be loaded")


def post_message(name, **kwargs):
    """
    Send a message to a Slack channel.

    .. code-block:: yaml

        slack-message:
          slack.post_message:
            - channel: '#general'
            - from_name: SuperAdmin
            - message: 'This state was executed successfully.'
            - api_key: peWcBiMOS9HrZG15peWcBiMOS9HrZG15

    The following parameters are required:

    api_key parameters:
        name
            The unique name for this event.

        channel
            The channel to send the message to. Can either be the ID or the name.

        from_name
            The name of that is to be shown in the "from" field.

        message
            The message that is to be sent to the Slack channel.

        The following parameters are optional:

        api_key
            The api key for Slack to use for authentication,
            if not specified in the configuration options of master or minion.

        icon
            URL to an image to use as the icon for this message

    webhook parameters:
        name
            The unique name for this event.

        message
            The message that is to be sent to the Slack channel.

        color
            The color of border of left side

        short
            An optional flag indicating whether the value is short
            enough to be displayed side-by-side with other values.

        webhook
            The identifier of WebHook (URL or token).

        channel
            The channel to use instead of the WebHook default.

        username
            Username to use instead of WebHook default.

        icon_emoji
            Icon to use instead of WebHook default.
    """
    ret = {"name": name, "changes": {}, "result": False, "comment": ""}
    api_key = kwargs.get("api_key")
    webhook = kwargs.get("webhook")

    # If neither api_key and webhook are not provided at the CLI, check the config
    if not api_key and not webhook:
        api_key = __salt__["config.get"]("slack.api_key") or __salt__["config.get"](
            "slack:api_key"
        )
        if not api_key:
            webhook = __salt__["config.get"]("slack.hook") or __salt__["config.get"](
                "slack:hook"
            )
            if not webhook:
                ret["comment"] = "Please specify api_key or webhook."
                return ret

    if api_key and webhook:
        ret["comment"] = "Please specify only either api_key or webhook."
        return ret

    if api_key and not kwargs.get("channel"):
        ret["comment"] = "Slack channel is missing."
        return ret

    if api_key and not kwargs.get("from_name"):
        ret["comment"] = "Slack from name is missing."
        return ret

    if not kwargs.get("message"):
        ret["comment"] = "Slack message is missing."
        return ret

    if __opts__["test"]:
        ret["comment"] = "The following message is to be sent to Slack: {}".format(
            kwargs.get("message")
        )
        ret["result"] = None
        return ret

    try:
        if api_key:
            result = __salt__["slack.post_message"](
                channel=kwargs.get("channel"),
                message=kwargs.get("message"),
                from_name=kwargs.get("from_name"),
                api_key=kwargs.get("api_key"),
                icon=kwargs.get("icon"),
            )
        elif webhook:
            result = __salt__["slack.call_hook"](
                message=kwargs.get("message"),
                attachment=kwargs.get("attachment"),
                color=kwargs.get("color", "good"),
                short=kwargs.get("short"),
                identifier=kwargs.get("webhook"),
                channel=kwargs.get("channel"),
                username=kwargs.get("username"),
                icon_emoji=kwargs.get("icon_emoji"),
            )

    except SaltInvocationError as sie:
        ret["comment"] = f"Failed to send message ({sie}): {name}"
    else:
        if isinstance(result, bool) and result:
            ret["result"] = True
            ret["comment"] = f"Sent message: {name}"
        else:
            ret["comment"] = "Failed to send message ({}): {}".format(
                result["message"], name
            )

    return ret



# Copyright (c) 2018 Cisco and/or its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Manage NX-OS System Image Upgrades.

.. versionadded:: 3001

:maturity:   new
:platform:   nxos
:codeauthor: Michael G Wiebe

For documentation on setting up the nxos proxy minion look in the documentation
for :mod:`salt.proxy.nxos<salt.proxy.nxos>`.
"""

import logging

__virtualname__ = "nxos"
__virtual_aliases__ = ("nxos_upgrade",)

log = logging.getLogger(__name__)


def __virtual__():
    return __virtualname__


def image_running(name, system_image, kickstart_image=None, issu=True, **kwargs):
    """
    Ensure the NX-OS system image is running on the device.

    name
        Name of the salt state task

    system_image
        Name of the system image file on bootflash:

    kickstart_image
        Name of the kickstart image file on bootflash:
        This is not needed if the system_image is a combined system and
        kickstart image
        Default: None

    issu
        Ensure the correct system is running on the device using an in service
        software upgrade, or force a disruptive upgrade by setting the option
        to False.
        Default: False

    timeout
        Timeout in seconds for long running 'install all' upgrade command.
        Default: 900

    Examples:

    .. code-block:: yaml

        upgrade_software_image_n9k:
          nxos.image_running:
            - name: Ensure nxos.7.0.3.I7.5a.bin is running
            - system_image: nxos.7.0.3.I7.5a.bin
            - issu: True

        upgrade_software_image_n7k:
          nxos.image_running:
            - name: Ensure n7000-s2-kickstart.8.0.1.bin is running
            - kickstart_image: n7000-s2-kickstart.8.0.1.bin
            - system_image: n7000-s2-dk9.8.0.1.bin
            - issu: False
    """
    ret = {"name": name, "result": False, "changes": {}, "comment": ""}

    if kickstart_image is None:
        upgrade = __salt__["nxos.upgrade"](
            system_image=system_image, issu=issu, **kwargs
        )
    else:
        upgrade = __salt__["nxos.upgrade"](
            system_image=system_image,
            kickstart_image=kickstart_image,
            issu=issu,
            **kwargs,
        )

    if upgrade["upgrade_in_progress"]:
        ret["result"] = upgrade["upgrade_in_progress"]
        ret["changes"] = upgrade["module_data"]
        ret["comment"] = "NX-OS Device Now Being Upgraded - See Change Details Below"
    elif upgrade["succeeded"]:
        ret["result"] = upgrade["succeeded"]
        ret["comment"] = f"NX-OS Device Running Image: {_version_info()}"
    else:
        ret["comment"] = "Upgrade Failed: {}.".format(upgrade["error_data"])

    return ret


def _version_info():
    """
    Helper method to return running image version
    """
    if "NXOS" in __grains__["nxos"]["software"]:
        return __grains__["nxos"]["software"]["NXOS"]
    elif "kickstart" in __grains__["nxos"]["software"]:
        return __grains__["nxos"]["software"]["kickstart"]
    else:
        return "Unable to detect sofware version"



import logging

import salt.client.netapi
from tests.support.mock import Mock, patch


def test_run_log(caplog, master_opts):
    """
    test salt.client.netapi logs correct message
    """
    master_opts["rest_cherrypy"] = {"port": 8000}
    mock_process = Mock()
    mock_process.add_process.return_value = True
    patch_process = patch.object(salt.utils.process, "ProcessManager", mock_process)
    with caplog.at_level(logging.INFO):
        with patch_process:
            netapi = salt.client.netapi.NetapiClient(master_opts)
            netapi.run()
    assert "Starting RunNetapi(salt.loaded.int.netapi.rest_cherrypy)" in caplog.text


def test_run_netapi_can_take_process_kwargs():
    salt.client.netapi.RunNetapi({}, "fname", name="name")



"""
Tests for the spm build utility
"""

import os
import shutil

import pytest

import salt.utils.files
import salt.utils.path
from tests.support.case import ModuleCase, SPMCase


@pytest.mark.windows_whitelisted
@pytest.mark.destructive_test
class SPMBuildTest(SPMCase, ModuleCase):
    """
    Validate the spm build command
    """

    def setUp(self):
        self.config = self._spm_config()
        self._spm_build_files(self.config)

    def test_spm_build(self):
        """
        test spm build
        """
        self.run_spm("build", self.config, self.formula_dir)
        spm_file = os.path.join(self.config["spm_build_dir"], "apache-201506-2.spm")
        # Make sure .spm file gets created
        self.assertTrue(os.path.exists(spm_file))
        # Make sure formula path dir is created
        self.assertTrue(os.path.isdir(self.config["formula_path"]))

    @pytest.mark.skip_if_binaries_missing("fallocate")
    @pytest.mark.slow_test
    @pytest.mark.timeout_unless_on_windows(120)
    def test_spm_build_big_file(self):
        """
        test spm build with a big file
        """
        # check to make sure there is enough space to run this test
        check_space = self.run_function("status.diskusage", ["/tmp"])
        space = check_space["/tmp"]["available"]
        if space < 3000000000:
            self.skipTest("Not enough space on host to run this test")

        self.run_function(
            "cmd.run",
            [
                "fallocate -l 1G {}".format(
                    os.path.join(self.formula_sls_dir, "bigfile.txt")
                )
            ],
        )
        self.run_spm("build", self.config, self.formula_dir)
        spm_file = os.path.join(self.config["spm_build_dir"], "apache-201506-2.spm")
        self.run_spm("install", self.config, spm_file)

        get_files = self.run_spm("files", self.config, "apache")

        files = ["apache.sls", "bigfile.txt"]
        for sls in files:
            self.assertIn(sls, " ".join(get_files))

    @pytest.mark.slow_test
    def test_spm_build_exclude(self):
        """
        test spm build while excluding directory
        """
        git_dir = os.path.join(self.formula_sls_dir, ".git")
        os.makedirs(git_dir)
        files = ["donotbuild1", "donotbuild2", "donotbuild3"]

        for git_file in files:
            with salt.utils.files.fopen(os.path.join(git_dir, git_file), "w") as fp:
                fp.write("Please do not include me in build")

        self.run_spm("build", self.config, self.formula_dir)
        spm_file = os.path.join(self.config["spm_build_dir"], "apache-201506-2.spm")
        self.run_spm("install", self.config, spm_file)

        get_files = self.run_spm("files", self.config, "apache")

        for git_file in files:
            self.assertNotIn(git_file, " ".join(get_files))

    def tearDown(self):
        shutil.rmtree(self._tmp_spm)



"""
Management of Zabbix host groups.

:codeauthor: Jiri Kotlin <jiri.kotlin@ultimum.io>


"""


def __virtual__():
    """
    Only make these states available if Zabbix module is available.
    """
    if "zabbix.hostgroup_create" in __salt__:
        return True
    return (False, "zabbix module could not be loaded")


def present(name, **kwargs):
    """
    Ensures that the host group exists, eventually creates new host group.

    .. versionadded:: 2016.3.0

    :param name: name of the host group
    :param _connection_user: Optional - zabbix user (can also be set in opts or pillar, see module's docstring)
    :param _connection_password: Optional - zabbix password (can also be set in opts or pillar, see module's docstring)
    :param _connection_url: Optional - url of zabbix frontend (can also be set in opts, pillar, see module's docstring)

    .. code-block:: yaml

        create_testing_host_group:
            zabbix_hostgroup.present:
                - name: 'My hostgroup name'


    """
    connection_args = {}
    if "_connection_user" in kwargs:
        connection_args["_connection_user"] = kwargs["_connection_user"]
    if "_connection_password" in kwargs:
        connection_args["_connection_password"] = kwargs["_connection_password"]
    if "_connection_url" in kwargs:
        connection_args["_connection_url"] = kwargs["_connection_url"]
    ret = {"name": name, "changes": {}, "result": False, "comment": ""}

    # Comment and change messages
    comment_hostgroup_created = f"Host group {name} created."
    comment_hostgroup_notcreated = f"Unable to create host group: {name}. "
    comment_hostgroup_exists = f"Host group {name} already exists."
    changes_hostgroup_created = {
        name: {
            "old": f"Host group {name} does not exist.",
            "new": f"Host group {name} created.",
        }
    }

    hostgroup_exists = __salt__["zabbix.hostgroup_exists"](name, **connection_args)

    # Dry run, test=true mode
    if __opts__["test"]:
        if hostgroup_exists:
            ret["result"] = True
            ret["comment"] = comment_hostgroup_exists
        else:
            ret["result"] = None
            ret["comment"] = comment_hostgroup_created
            ret["changes"] = changes_hostgroup_created
        return ret

    if hostgroup_exists:
        ret["result"] = True
        ret["comment"] = comment_hostgroup_exists
    else:
        hostgroup_create = __salt__["zabbix.hostgroup_create"](name, **connection_args)

        if "error" not in hostgroup_create:
            ret["result"] = True
            ret["comment"] = comment_hostgroup_created
            ret["changes"] = changes_hostgroup_created
        else:
            ret["result"] = False
            ret["comment"] = comment_hostgroup_notcreated + str(
                hostgroup_create["error"]
            )

    return ret


def absent(name, **kwargs):
    """
    Ensures that the host group does not exist, eventually delete host group.

    .. versionadded:: 2016.3.0

    :param name: name of the host group
    :param _connection_user: Optional - zabbix user (can also be set in opts or pillar, see module's docstring)
    :param _connection_password: Optional - zabbix password (can also be set in opts or pillar, see module's docstring)
    :param _connection_url: Optional - url of zabbix frontend (can also be set in opts, pillar, see module's docstring)

    .. code-block:: yaml

        delete_testing_host_group:
            zabbix_hostgroup.absent:
                - name: 'My hostgroup name'

    """

    ret = {"name": name, "changes": {}, "result": False, "comment": ""}

    # Comment and change messages
    comment_hostgroup_deleted = f"Host group {name} deleted."
    comment_hostgroup_notdeleted = f"Unable to delete host group: {name}. "
    comment_hostgroup_notexists = f"Host group {name} does not exist."
    changes_hostgroup_deleted = {
        name: {
            "old": f"Host group {name} exists.",
            "new": f"Host group {name} deleted.",
        }
    }

    connection_args = {}
    if "_connection_user" in kwargs:
        connection_args["_connection_user"] = kwargs["_connection_user"]
    if "_connection_password" in kwargs:
        connection_args["_connection_password"] = kwargs["_connection_password"]
    if "_connection_url" in kwargs:
        connection_args["_connection_url"] = kwargs["_connection_url"]

    hostgroup_exists = __salt__["zabbix.hostgroup_exists"](name, **connection_args)

    # Dry run, test=true mode
    if __opts__["test"]:
        if not hostgroup_exists:
            ret["result"] = True
            ret["comment"] = comment_hostgroup_notexists
        else:
            ret["result"] = None
            ret["comment"] = comment_hostgroup_deleted
            ret["changes"] = changes_hostgroup_deleted
        return ret

    hostgroup_get = __salt__["zabbix.hostgroup_get"](name, **connection_args)

    if not hostgroup_get:
        ret["result"] = True
        ret["comment"] = comment_hostgroup_notexists
    else:
        try:
            groupid = hostgroup_get[0]["groupid"]
            hostgroup_delete = __salt__["zabbix.hostgroup_delete"](
                groupid, **connection_args
            )
        except KeyError:
            hostgroup_delete = False

        if hostgroup_delete and "error" not in hostgroup_delete:
            ret["result"] = True
            ret["comment"] = comment_hostgroup_deleted
            ret["changes"] = changes_hostgroup_deleted
        else:
            ret["result"] = False
            ret["comment"] = comment_hostgroup_notdeleted + str(
                hostgroup_delete["error"]
            )

    return ret



"""
    :codeauthor: `Nitin Madhok <nmadhok@g.clemson.edu>`

    tests.unit.cloud.clouds.vmware_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""

import pytest

from salt.cloud.clouds import vmware
from salt.exceptions import SaltCloudSystemExit
from tests.support.mock import MagicMock

# Attempt to import pyVim and pyVmomi libs
HAS_LIBS = True
# pylint: disable=import-error,no-name-in-module,unused-import
try:
    from pyVim.connect import Disconnect, SmartConnect
    from pyVmomi import vim, vmodl
except ImportError:
    HAS_LIBS = False
# pylint: enable=import-error,no-name-in-module,unused-import


def _test_clone_type(clone_type):
    """
    Assertions for checking that a certain clone type
    works
    """
    obj_ref = MagicMock()
    obj_ref.snapshot = vim.vm.Snapshot(None, None)
    obj_ref.snapshot.currentSnapshot = vim.vm.Snapshot(None, None)
    clone_spec = vmware.handle_snapshot(
        vim.vm.ConfigSpec(),
        obj_ref,
        vim.vm.RelocateSpec(),
        False,
        {"snapshot": {"disk_move_type": clone_type}},
    )
    assert clone_spec.location.diskMoveType == clone_type

    obj_ref2 = MagicMock()
    obj_ref2.snapshot = vim.vm.Snapshot(None, None)
    obj_ref2.snapshot.currentSnapshot = vim.vm.Snapshot(None, None)

    clone_spec2 = vmware.handle_snapshot(
        vim.vm.ConfigSpec(),
        obj_ref2,
        vim.vm.RelocateSpec(),
        True,
        {"snapshot": {"disk_move_type": clone_type}},
    )

    assert clone_spec2.location.diskMoveType == clone_type


@pytest.mark.skipif(
    HAS_LIBS is False, reason="Install pyVmomi to be able to run this unit test."
)
def test_quick_linked_clone():
    """
    Test that disk move type is
    set to createNewChildDiskBacking
    """
    _test_clone_type(vmware.QUICK_LINKED_CLONE)


@pytest.mark.skipif(
    HAS_LIBS is False, reason="Install pyVmomi to be able to run this unit test."
)
def test_current_state_linked_clone():
    """
    Test that disk move type is
    set to moveChildMostDiskBacking
    """
    _test_clone_type(vmware.CURRENT_STATE_LINKED_CLONE)


@pytest.mark.skipif(
    HAS_LIBS is False, reason="Install pyVmomi to be able to run this unit test."
)
def test_copy_all_disks_full_clone():
    """
    Test that disk move type is
    set to moveAllDiskBackingsAndAllowSharing
    """
    _test_clone_type(vmware.COPY_ALL_DISKS_FULL_CLONE)


@pytest.mark.skipif(
    HAS_LIBS is False, reason="Install pyVmomi to be able to run this unit test."
)
def test_flatten_all_all_disks_full_clone():
    """
    Test that disk move type is
    set to moveAllDiskBackingsAndDisallowSharing
    """
    _test_clone_type(vmware.FLATTEN_DISK_FULL_CLONE)


@pytest.mark.skipif(
    HAS_LIBS is False, reason="Install pyVmomi to be able to run this unit test."
)
def test_raises_error_for_invalid_disk_move_type():
    """
    Test that invalid disk move type
    raises error
    """
    pytest.raises(SaltCloudSystemExit, _test_clone_type, "foobar")
